@@ -16,7 +16,7 @@
 
 		int mw32_BDF_TextOut(HDC hdc, int left,
 						int top, unsigned char *text, int dim, int bytelen,
-						int fixed_pitch_size);
+						int fixed_pitch_size, int char_space, int line_space);
 
 ******************************************/
 #include <windows.h>
@@ -35,6 +35,220 @@ void mw32_free_bdf_font(bdffont *fontp);
 bdffont *mw32_init_bdf_font(char *filename);
 static int compchr(char *text, int len, char ch);
 
+#define GLYPH_CACHE		1
+#define BITMAP_CACHE	1
+
+#if GLYPH_CACHE
+static struct gcache {
+	int					index;
+	unsigned int		hit;
+	glyph_struct		glyph;
+} glyph_cache[0x500];
+
+static VOID
+init_cached_glyph()
+{
+	int				i;
+
+	for (i = 0; i < sizeof(glyph_cache) / sizeof(struct gcache); i++)
+	{
+		if (glyph_cache[i].glyph.bitmap != NULL)
+			free(glyph_cache[i].glyph.bitmap);
+	}
+	memset(glyph_cache, 0, sizeof(glyph_cache));
+}
+
+static int
+get_cached_glyph(bdffont *fontp, int index, int size, glyph_struct *glyph)
+{
+	int				i;
+	unsigned int	min = 0xffffffff;
+	int				array = -1;
+
+	if (0x20 <= index && index <= 0x7f)				/* base 0x0000 */
+	{
+		array = index - 0x20;
+		if (glyph_cache[array].hit)
+			goto glyph_copy;
+	}
+	else if (0xa0 <= index && index <= 0xdf)		/* base 0x0060 */
+	{
+		array = index - 0xa0 + 0x60;
+		if (glyph_cache[array].hit)
+			goto glyph_copy;
+	}
+	else if (0x2120 <= index && index <= 0x217f)	/* base 0x00a0 */
+	{
+		array = index - 0x2120 + 0xa0;
+		if (glyph_cache[array].hit)
+			goto glyph_copy;
+	}
+	else if (0x2330 <= index && index <= 0x237f)	/* base 0x0100 */
+	{
+		array = index - 0x2330 + 0x100;
+		if (glyph_cache[array].hit)
+			goto glyph_copy;
+	}
+	else if (0x2420 <= index && index <= 0x247f)	/* base 0x0150 */
+	{
+		array = index - 0x2420 + 0x150;
+		if (glyph_cache[array].hit)
+			goto glyph_copy;
+	}
+	else if (0x2520 <= index && index <= 0x257f)	/* base 0x01b0 */
+	{
+		array = index - 0x2520 + 0x1b0;
+		if (glyph_cache[array].hit)
+			goto glyph_copy;
+	}
+	else											/* base 0x0210 */
+	{
+		for (i = 0x210; i < sizeof(glyph_cache) / sizeof(struct gcache); i++)
+		{
+			if (glyph_cache[i].index == index)
+			{
+				array = i;
+				goto glyph_copy;
+			}
+			if (glyph_cache[i].index == 0)
+			{
+				array = i;
+				break;
+			}
+			if (min > glyph_cache[i].hit)
+			{
+				min = glyph_cache[i].hit;
+				array = i;
+			}
+		}
+	}
+	if (array < 0)
+		array = 0x210;
+	glyph_cache[array].index = 0;
+	glyph_cache[array].hit   = 0;
+	if (glyph_cache[array].glyph.bitmap == NULL)
+	{
+		glyph_cache[array].glyph.bitmap
+					= (unsigned char*) malloc(sizeof(unsigned char) * size);
+		if (glyph_cache[array].glyph.bitmap == NULL)
+			return(0);
+	}
+	if (mw32_get_bdf_glyph(fontp, index, size, &glyph_cache[array].glyph))
+	{
+glyph_copy:
+		if (glyph_cache[array].hit < 0xf0000000)
+			glyph_cache[array].hit++;
+		glyph->dwidth		= glyph_cache[array].glyph.dwidth;
+		glyph->bbw			= glyph_cache[array].glyph.bbw;
+		glyph->bbh			= glyph_cache[array].glyph.bbh;
+		glyph->bbox			= glyph_cache[array].glyph.bbox;
+		glyph->bboy			= glyph_cache[array].glyph.bboy;
+		glyph->bitmap_size	= glyph_cache[array].glyph.bitmap_size;
+		memcpy(glyph->bitmap, glyph_cache[array].glyph.bitmap, size);
+		return(1);
+	}
+	return(0);
+}
+#endif
+
+#if BITMAP_CACHE
+static struct bcache {
+	int					index;
+	unsigned int		hit;
+	HBITMAP				hBitmap;
+} bitmap_cache[0x500];
+
+static VOID
+init_cached_bitmap()
+{
+	int				i;
+
+	for (i = 0; i < sizeof(bitmap_cache) / sizeof(struct bcache); i++)
+	{
+		if (bitmap_cache[i].hBitmap != NULL)
+			DeleteObject(bitmap_cache[i].hBitmap);
+	}
+	memset(bitmap_cache, 0, sizeof(bitmap_cache));
+}
+
+static HBITMAP
+get_cached_bitmap(int index, glyph_struct *glyph)
+{
+	HBITMAP			hBMP;
+	int				i;
+	unsigned int	min = 0xffffffff;
+	int				array = -1;
+
+	if (0x20 <= index && index <= 0x7f)				/* base 0x0000 */
+	{
+		array = index - 0x20;
+		if (bitmap_cache[array].hBitmap)
+			return(bitmap_cache[array].hBitmap);
+	}
+	else if (0xa0 <= index && index <= 0xdf)		/* base 0x0060 */
+	{
+		array = index - 0xa0 + 0x60;
+		if (bitmap_cache[array].hBitmap)
+			return(bitmap_cache[array].hBitmap);
+	}
+	else if (0x2120 <= index && index <= 0x217f)	/* base 0x00a0 */
+	{
+		array = index - 0x2120 + 0xa0;
+		if (bitmap_cache[array].hBitmap)
+			return(bitmap_cache[array].hBitmap);
+	}
+	else if (0x2330 <= index && index <= 0x237f)	/* base 0x0100 */
+	{
+		array = index - 0x2330 + 0x100;
+		if (bitmap_cache[array].hBitmap)
+			return(bitmap_cache[array].hBitmap);
+	}
+	else if (0x2420 <= index && index <= 0x247f)	/* base 0x0150 */
+	{
+		array = index - 0x2420 + 0x150;
+		if (bitmap_cache[array].hBitmap)
+			return(bitmap_cache[array].hBitmap);
+	}
+	else if (0x2520 <= index && index <= 0x257f)	/* base 0x01b0 */
+	{
+		array = index - 0x2520 + 0x1b0;
+		if (bitmap_cache[array].hBitmap)
+			return(bitmap_cache[array].hBitmap);
+	}
+	else											/* base 0x0210 */
+	{
+		for (i = 0x210; i < sizeof(bitmap_cache) / sizeof(struct bcache); i++)
+		{
+			if (bitmap_cache[i].index == index)
+			{
+				if (bitmap_cache[i].hit < 0xf0000000)
+					bitmap_cache[i].hit++;
+				return(bitmap_cache[i].hBitmap);
+			}
+			if (bitmap_cache[i].index == 0)
+			{
+				array = i;
+				break;
+			}
+			if (min > bitmap_cache[i].hit)
+			{
+				min = bitmap_cache[i].hit;
+				array = i;
+			}
+		}
+	}
+	if (array < 0)
+		array = 0x210;
+	if (bitmap_cache[array].hBitmap)
+		DeleteObject(bitmap_cache[array].hBitmap);
+	hBMP = CreateBitmap(glyph->bbw, glyph->bbh, 1, 1, glyph->bitmap);
+	bitmap_cache[array].index = index;
+	bitmap_cache[array].hit   = 1;
+	bitmap_cache[array].hBitmap = hBMP;
+	return(hBMP);
+}
+#endif
+
 static int 
 search_file_line(char *key, char *start, int len, char **val, char **next)
 {
@@ -188,8 +402,8 @@ mw32_init_bdf_font(char *filename)
 	font = MapViewOfFile(hfilemap, FILE_MAP_READ, 0, 0, 0);
 	if (!font)
 	{
-		CloseHandle(hfile);
 		CloseHandle(hfilemap);
+		CloseHandle(hfile);
 		return NULL;
 	}
 
@@ -216,7 +430,7 @@ mw32_free_bdf_font(bdffont *fontp)
 {
 	int i;
 
-	UnmapViewOfFile(fontp->hfilemap);
+	UnmapViewOfFile(fontp->font);
 	CloseHandle(fontp->hfilemap);
 	CloseHandle(fontp->hfile);
 	free(fontp->filename);
@@ -379,7 +593,7 @@ mw32_get_bdf_glyph(bdffont *fontp, int index, int size, glyph_struct *glyph)
 int
 mw32_BDF_TextOut(HDC hdc, int left,
 		 int top, unsigned char *text, int bytelen, int fixed_pitch_size,
-		 int bitmap, int rev, DWORD fg, DWORD bg)
+		 int bitmap, int rev, DWORD fg, DWORD bg, int cspace, int lspace)
 {
 	int bitmap_size, index, btop;
 	unsigned char *textp;
@@ -477,7 +691,11 @@ mw32_BDF_TextOut(HDC hdc, int left,
 				left += glyph.dwidth;
 			continue;
 		}
+#if GLYPH_CACHE
+		if (!get_cached_glyph(fontp, index, bitmap_size, &glyph))
+#else
 		if (!mw32_get_bdf_glyph(fontp, index, bitmap_size, &glyph))
+#endif
 		{
 #if 1
 			if (kanji)
@@ -503,7 +721,11 @@ mw32_BDF_TextOut(HDC hdc, int left,
 			return 0;
 #endif
 		}
+#if BITMAP_CACHE
+		hBMP = get_cached_bitmap(index, &glyph);
+#else
 		hBMP = CreateBitmap(glyph.bbw, glyph.bbh, 1, 1, glyph.bitmap);
+#endif
 		if (textalign & TA_BASELINE)
 		{
 			btop = top - (glyph.bbh + glyph.bboy);
@@ -517,10 +739,22 @@ mw32_BDF_TextOut(HDC hdc, int left,
 			btop = top;
 		}
 
+#if BITMAP_CACHE
+		if (!horgobj)
+		{
+			horgobj = SelectObject(hCompatDC, hBMP);
+		}
+		else
+		{
+			SelectObject(hCompatDC, horgobj);
+			SelectObject(hCompatDC, hBMP);
+		}
+#else
 		if (horgobj)
 		{
-			SelectObject(hCompatDC, hBMP);
+			SelectObject(hCompatDC, horgobj);
 			DeleteObject(holdobj);
+			SelectObject(hCompatDC, hBMP);
 			holdobj = hBMP;
 		}
 		else
@@ -528,9 +762,10 @@ mw32_BDF_TextOut(HDC hdc, int left,
 			horgobj = SelectObject(hCompatDC, hBMP);
 			holdobj = hBMP;
 		}
+#endif
 		if (!bitmap)
 		{
-			if (!g_fix)
+			if (g_fix || cspace || lspace)
 			{
 				HBRUSH		hbrush, holdbrush;
 				RECT		rcWindow;
@@ -539,17 +774,17 @@ mw32_BDF_TextOut(HDC hdc, int left,
 				holdbrush = SelectObject(hdc, hbrush);
 				rcWindow.left = left;
 				rcWindow.top = btop;
-				rcWindow.right = left + g_xsize * (index > 0xff ? 2 : 1);
-				rcWindow.bottom = btop + g_ysize;
+				rcWindow.right = left + (g_xsize + cspace) * (index > 0xff ? 2 : 1);
+				rcWindow.bottom = btop + g_ysize + lspace;
 				FillRect(hdc, &rcWindow, hbrush);
 				SelectObject(hdc, holdbrush);
 				DeleteObject(hbrush);
 			}
 			BitBlt(hdc, left, btop, glyph.bbw, glyph.bbh, hCompatDC, 0, 0, SRCCOPY);
 		}
+#if 0
 		else if (index == ' ' && compchr(textp, bytelen, ' '))
 			BitBlt(hdc, left, btop, glyph.bbw, glyph.bbh, hCompatDC, 0, 0, SRCAND);
-#if 0
 		else if (rev)
 			BitBlt(hdc, left, btop, glyph.bbw, glyph.bbh, hCompatDC, 0, 0, SRCAND);
 		else
@@ -568,7 +803,9 @@ mw32_BDF_TextOut(HDC hdc, int left,
 	SelectObject(hCompatDC, horgobj);
 	SelectObject(hdc, hOrgBrush);
 	DeleteObject(hFgBrush);
+#if !BITMAP_CACHE
 	DeleteObject(hBMP);
+#endif
 	DeleteDC(hCompatDC);
 	RestoreDC(hdc, -1);
 	free(glyph.bitmap);
@@ -577,9 +814,9 @@ mw32_BDF_TextOut(HDC hdc, int left,
 }
 
 int
-bdfTextOut(HDC hdc, int left, int top, UINT opt, CONST RECT *lprc, char *text, UINT size, CONST INT * lpdx, int bitmap, int rev, DWORD fg, DWORD bg)
+bdfTextOut(HDC hdc, int left, int top, UINT opt, CONST RECT *lprc, char *text, UINT size, CONST INT * lpdx, int bitmap, int rev, DWORD fg, DWORD bg, int cspace, int lspace)
 {
-	return mw32_BDF_TextOut(hdc, left, top, text, size, *lpdx, bitmap, rev, fg, bg);
+	return mw32_BDF_TextOut(hdc, left, top, text, size, *lpdx, bitmap, rev, fg, bg, cspace, lspace);
 }
 
 /*------------------------------------------------------------------------------
@@ -636,7 +873,7 @@ BDFHookProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 						g_jbdffontp = bdffontp;
 						mw32_BDF_TextOut(hdc, rt.left, rt.top,
 								"A@F", 12, bdffontp->urx / 2,
-								FALSE, FALSE, RGB(0, 0, 0), RGB(185, 185, 185));
+								FALSE, FALSE, RGB(0, 0, 0), RGB(185, 185, 185), 0, 0);
 						g_jbdffontp = NULL;
 					}
 					else
@@ -644,7 +881,7 @@ BDFHookProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 						g_bdffontp = bdffontp;
 						mw32_BDF_TextOut(hdc, rt.left, rt.top,
 								"AaBbYyZz", 8, bdffontp->urx,
-								FALSE, FALSE, RGB(0, 0, 0), RGB(185, 185, 185));
+								FALSE, FALSE, RGB(0, 0, 0), RGB(185, 185, 185), 0, 0);
 						g_bdffontp = NULL;
 					}
 					ReleaseDC(hwndBrush, hdc);
@@ -931,6 +1168,12 @@ GetBDFfont(HINSTANCE hInst, int mode, char *fname, char *jfname, int *xsize, int
 		if (DialogBoxParam(hInst, "BDF", NULL, (DLGPROC)DialogProc, (LPARAM)config) != 0 && !(*config))
 			return; 
 	}
+#if GLYPH_CACHE
+	init_cached_glyph();
+#endif
+#if BITMAP_CACHE
+	init_cached_bitmap();
+#endif
 	/* check */
 	g_bdffontp  = mw32_init_bdf_font(g_bdffile);
 	g_jbdffontp = mw32_init_bdf_font(g_jbdffile);
@@ -958,6 +1201,8 @@ GetBDFfont(HINSTANCE hInst, int mode, char *fname, char *jfname, int *xsize, int
 		*ysize = ay;
 	else
 		*ysize = jy;
+	if (!g_fix)
+		g_fix = ay == jy ? 1 : 0;
 	g_xsize = *xsize;
 	g_ysize = *ysize;
 	return;
