@@ -42,4 +42,4 @@ extern void mw32_free_bdf_font(bdffont *fontp);
 extern int mw32_get_bdf_glyph(bdffont *fontp, int index, int size,
 			      glyph_struct *glyph);
 extern int mw32_BDF_TextOut(HDC hdc, int left,
-			    int top, unsigned char *text, int bytelen, int fixed_pitch_size, int bitmap, int reverse, DWORD fg, DWORD bg);
+			    int top, unsigned char *text, int bytelen, int fixed_pitch_size, int bitmap, int reverse, DWORD fg, DWORD bg, int cspace, int lspace);
