@@ -90,8 +90,8 @@ open_buffer()
 			{
 				if (*p == '.')
 					dotp = p;
-# if defined(KANJI) && defined(MSDOS)
-				if (ISkanji(*p))
+# ifdef KANJI
+				else if (ISkanji(*p))
 					p++;
 # endif
 				p++;
@@ -99,7 +99,7 @@ open_buffer()
 			if (dotp == curbuf->b_filename)
 				dotp = NULL;
 			STRCPY(tmp, SYSVIMRC_FILE);
-			if (dotp)
+			if (dotp != NULL)
 				STRCAT(tmp, dotp);
 			if (dosource(tmp) == FAIL && dotp != NULL)
 				(void)dosource((char_u *)SYSVIMRC_FILE);
@@ -108,7 +108,7 @@ open_buffer()
 				if (p_exrc)
 				{
 					STRCPY(tmp, VIMRC_FILE);
-					if (dotp)
+					if (dotp != NULL)
 						STRCAT(tmp, dotp);
 # ifdef UNIX
 					{
@@ -188,6 +188,9 @@ close_buffer(buf, free_buf, remove)
 #endif
 #ifdef TRACK
 		free(buf->b_p_trs);
+#endif
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+		syn_clr(buf);
 #endif
 		free(buf);
 	}
@@ -384,7 +387,20 @@ enter_buffer(buf)
 
 	if (buf->b_neverloaded)
 	{
+#if 1
+		BUF			*wbuf;
+
+		for (wbuf = firstbuf; wbuf != NULL; wbuf = wbuf->b_next)
+		{
+			if (wbuf == curbuf)
+			{
+				buf_copy_options(curbuf, buf);
+				break;
+			}
+		}
+#else
 		buf_copy_options(curbuf, buf);
+#endif
 		buf->b_neverloaded = FALSE;
 	}
 	curwin->w_buffer = buf;
@@ -919,9 +935,9 @@ fileinfo(fullname)
 #ifdef KANJI
 	{
 		char_u	*p		= ml_get(curwin->w_cursor.lnum);
-		int		pcol	= getvcol(curwin, &curwin->w_cursor, 3),
-				lastcol = STRLEN(p),
-				code	= *ml_get_cursor() & 0xff;
+		int		pcol	= getvcol(curwin, &curwin->w_cursor, 3) + 1;
+		int		lastcol = STRLEN(p);
+		int		code	= *ml_get_cursor() & 0xff;
 		FPOS	last;
 		char_u	k1, k2;
 
@@ -934,15 +950,16 @@ fileinfo(fullname)
 			k2 = p[curwin->w_cursor.col+1];
 			kanjito(&k1, &k2, toupper(*curbuf->b_p_jc));
 			code = (k1 << 8) | k2;
+			--pcol;
 		}
-#ifdef UCODE
+# ifdef UCODE
 		else if (toupper(*curbuf->b_p_jc) == JP_WIDE)	/* UNICODE */
 		{
 			k1 = code;
 			kanato(&k1, &k2, toupper(*curbuf->b_p_jc));
 			code = (k1 << 8) | k2;
 		}
-#endif
+# endif
 		else if (ISkana(code))
 		{
 			k1 = code;
@@ -1042,9 +1059,9 @@ resettitle()
 	char_u	*	i = NULL;
 
 	if (lasttitle != NULL)
-		t = kanjiconvsto(lasttitle, toupper(JP_DISP));
+		t = kanjiconvsto(lasttitle, toupper(JP_DISP), TRUE);
 	if (lasticon != NULL)
-		i = kanjiconvsto(lasticon, toupper(JP_DISP));
+		i = kanjiconvsto(lasticon, toupper(JP_DISP), TRUE);
 	mch_settitle(t, i);
 	if (t != NULL)
 		free(t);
