@@ -437,7 +437,7 @@ do_esc:
 					continue;
 
 				i = hiscnt;
-			
+
 					/* save current command string */
 				if (c == K_SUARROW || c == K_SDARROW)
 				{
@@ -727,7 +727,7 @@ ccheck_abbr(c)
 {
 	if (p_paste || no_abbr)			/* no abbreviations or in paste mode */
 		return FALSE;
-	
+
 	return check_abbr(c, cmdbuff, cmdpos, 0);
 }
 
@@ -826,6 +826,7 @@ DoOneCmd(buff)
 	int					amount = 0;				/* for ":>" and ":<"; init for gcc */
 #ifdef KANJI
 	char_u				code;
+	int					ubig;
 #endif
 
 	if (quitmore)
@@ -1238,6 +1239,11 @@ DoOneCmd(buff)
 			{
 				if (check_fname() == FAIL)
 					goto doend;
+#ifndef notdef
+				if (curbuf->b_filename)
+					q = curbuf->b_filename;
+				else
+#endif
 				q = curbuf->b_xfilename;
 				n = 1;					/* length of what we expand */
 			}
@@ -1307,6 +1313,22 @@ DoOneCmd(buff)
 		}
 	}
 
+#ifdef NT
+	while ((usefilter || cmdidx == CMD_bang) && arg != NULL
+						&& (*arg == '.' || strchr(arg, '/') != NULL))
+	{
+		static char_u		argbuf[CMDBUFFSIZE];
+
+		if (strchr(arg, ' ') && (strchr(arg, ' ') < strchr(arg, '/')))
+			break;
+		if (strchr(arg, '\t') && (strchr(arg, '\t') < strchr(arg, '/')))
+			break;
+		if (FullName(arg, argbuf, sizeof(argbuf)) == OK)
+			arg = argbuf;
+		break;
+	}
+#endif
+
 /*
  * 6. switch on command name
  */
@@ -1387,7 +1409,7 @@ cmdswitch:
 				if (((cmdidx == CMD_wq ||
 						(curbuf->b_nwindows == 1 && curbuf->b_changed)) &&
 						(check_readonly() || dowrite(arg, FALSE) == FAIL)) ||
-					check_more(TRUE) == FAIL || 
+					check_more(TRUE) == FAIL ||
 					(firstwin == lastwin && check_changed_any(FALSE)))
 				{
 					exiting = FALSE;
@@ -1441,7 +1463,7 @@ cmdswitch:
 				ml_preserve(curbuf, TRUE);
 				break;
 
-		case CMD_args:		
+		case CMD_args:
 					/*
 					 * ":args file": handle like :next
 					 */
@@ -1766,6 +1788,7 @@ doargument:
 					break;
 #ifdef KANJI
 				code = *curbuf->b_p_jc;
+				ubig = curbuf->b_p_ubig;
 #endif
 				if (*arg == NUL)
 				{
@@ -1777,6 +1800,7 @@ doargument:
 					i = readfile(arg, NULL, line2, FALSE, (linenr_t)0, MAXLNUM);
 #ifdef KANJI
 				*curbuf->b_p_jc = code;
+				curbuf->b_p_ubig= ubig;
 #endif
 				if (i == FAIL)
 				{
@@ -1979,6 +2003,19 @@ doabbr:
 							break;
 				}
 				break;
+		case CMD_syntax:
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+				++RedrawingDisabled;
+				switch (syn_add(curwin->w_buffer, arg))
+				{
+					case 1: emsg(e_invarg);
+							break;
+					case 2: emsg(e_invstring);
+							break;
+				}
+				--RedrawingDisabled;
+#endif
+				break;
 
 		case CMD_display:
 				dodis();		/* display buffer contents */
@@ -2511,6 +2548,7 @@ doecmd(fname, sfname, command, hide, newlnum)
 	if (!oldbuf && !other_file && forceit)
 	{
 		char_u	code = *curbuf->b_p_jc;
+		int		ubig = curbuf->b_p_ubig;
 # ifdef FEXRC
 		int		fexrc = p_fexrc;
 		p_fexrc = FALSE;
@@ -2519,6 +2557,7 @@ doecmd(fname, sfname, command, hide, newlnum)
 		*curbuf->b_p_jc = tolower(*curbuf->b_p_jc);
 		(void)open_buffer();
 		*curbuf->b_p_jc = code;
+		curbuf->b_p_ubig= ubig;
 # ifdef FEXRC
 		p_fexrc = fexrc;
 # endif
@@ -2580,7 +2619,7 @@ getargcmd(argp)
 		}
 		if (*arg)
 			*arg++ = NUL;	/* terminate command with NUL */
-		
+
 		skipspace(&arg);	/* skip over spaces */
 		*argp = arg;
 	}
@@ -2655,7 +2694,7 @@ domake(arg)
 #endif
 }
 
-/* 
+/*
  * Redefine the argument list to 'str'.
  *
  * Return FAIL for failure, OK otherwise.
@@ -3168,8 +3207,13 @@ ExpandOne(str, list_notfound, mode)
 							if ((nextsetsuf = STRCHR(setsuf + 1, '.')) == NULL)
 								nextsetsuf = setsuf + STRLEN(setsuf);
 							setsuflen = (int)(nextsetsuf - setsuf);
+#ifdef MSDOS
+							if (filesuflen == setsuflen &&
+										vim_strnicmp(setsuf, filesuf, (size_t)setsuflen) == 0)
+#else
 							if (filesuflen == setsuflen &&
 										STRNCMP(setsuf, filesuf, (size_t)setsuflen) == 0)
+#endif
 								break;
 						}
 						if (*setsuf)				/* suffix matched: ignore file */
@@ -3530,16 +3574,21 @@ dosource(fname)
 #ifdef MSDOS
 	int				error = FALSE;
 #endif
-#ifdef KANJI
+#if defined(KANJI) || defined(FEXRC)
 	char_u			*p;
 #endif
+#ifdef FEXRC
+	int				do_fexrc = FALSE;
+	char_u		*	dotp = NULL;
+	char_u		*	q;
+#endif
 
 	expand_env(fname, NameBuff, MAXPATHL);		/* use NameBuff for expanded name */
 #ifdef KANJI
 	if ((p = gettail(NameBuff)) != NULL
 			&& (STRCMP(p, VIMRC_FILE) == 0
-					|| (STRNCMP(p, VIMRC_FILE, STRLEN(VIMRC_FILE))==0
-									&& p[STRLEN(VIMRC_FILE)] == '.')))
+					|| (STRNCMP(p, VIMRC_FILE, STRLEN(VIMRC_FILE)) == 0
+										&& p[STRLEN(VIMRC_FILE)] == '.')))
 	{
 		memmove(p + 2, p + 1, strlen(p));
 		p[1] = 'j';
@@ -3561,20 +3610,51 @@ dosource(fname)
 			else if (p[0] == '_')
 			{
 				*p = '.';
-				if ((fp = fopen(fileconvsto(NameBuff), READBIN)) == NULL)
-					return FAIL;
+				fp = fopen(fileconvsto(NameBuff), READBIN);
 			}
 # endif
-			else
-				return FAIL;
 		}
+		if (fp == NULL)
+			return FAIL;
 	}
+# ifdef DEFVIMRC_FILE
+	else if ((p = gettail(NameBuff)) != NULL
+									&& STRCMP(fname, DEFVIMRC_FILE) == 0)
+	{
+		memmove(p + 1, p, strlen(p) + 1);
+		p[0] = 'j';
+		if ((fp = fopen(fileconvsto(NameBuff), READBIN)) == NULL)
+		{
+			expand_env(fname, NameBuff, MAXPATHL);
+			fp = fopen(fileconvsto(NameBuff), READBIN);
+		}
+		if (fp == NULL)
+			return FAIL;
+	}
+# endif
 	else if ((fp = fopen(fileconvsto(NameBuff), READBIN)) == NULL)
 		return FAIL;
 #else
 	if ((fp = fopen((char *)NameBuff, READBIN)) == NULL)
 		return FAIL;
 #endif
+#ifdef FEXRC
+	if (p_fexrc && curbuf->b_filename != NULL)
+	{
+		STRCPY(NameBuff, curbuf->b_filename);
+		q = NameBuff;
+		while (*q)
+		{
+			if (*q == '.')
+				dotp = q;
+# ifdef KANJI
+			else if (ISkanji(*q))
+				q++;
+# endif
+			q++;
+		}
+	}
+#endif
 
 	++dont_sleep;			/* don't call sleep() in emsg() */
 	len = 0;
@@ -3606,12 +3686,84 @@ dosource(fname)
 		if (len >= 2)
 		{
 			char		tmp[IOSIZE];
+			int			w_jkc = p_jkc;
 
+			p_jkc = FALSE;
 			len = kanjiconvsfrom(IObuff, len, tmp, IOSIZE, NULL, JP_SYS, NULL);
 			tmp[len] = NUL;
-			docmdline((char_u *)tmp);
+			STRCPY(IObuff, tmp);
+			p_jkc = w_jkc;
+		}
+#endif
+#ifdef FEXRC
+		if (p_fexrc && IObuff[0] == '"' && IObuff[1] != '"')
+		{
+			p = &IObuff[1];
+			skipspace(&p);
+			if (!do_fexrc && dotp
+					&& (STRNCMP(p, "begin", strlen("begin")) == 0)
+					&& ((q = strstr(IObuff, "begin")) != NULL)
+					&& ((q = strstr(q, "suffixes")) != NULL))
+			{
+				char_u		c;
+
+				while (*q && (q = STRCHR(q, '.')) != NULL)
+				{
+					c = q[STRLEN(dotp)];
+# ifdef MSDOS
+					if ((c == NUL || c == '.' || c == ' ' || c == '\t')
+							&& vim_strnicmp(dotp, q, (size_t)STRLEN(dotp)) == 0)
+# else
+					if ((c == NUL || c == '.' || c == ' ' || c == '\t')
+										&& STRNCMP(dotp, q, STRLEN(dotp)) == 0)
+# endif
+					{
+						do_fexrc = TRUE;
+						break;
+					}
+					q++;
+				}
+			}
+			else if (do_fexrc && dotp
+					&& (STRNCMP(p, "end", strlen("end")) == 0)
+					&& ((q = strstr(IObuff, "end")) != NULL)
+					&& ((q = strstr(q, "suffixes")) != NULL))
+				do_fexrc = FALSE;
+			else if (!do_fexrc
+					&& curbuf->b_filename != NULL
+					&& (STRNCMP(p, "begin", strlen("begin")) == 0)
+					&& ((q = strstr(IObuff, "begin")) != NULL)
+					&& ((q = strstr(q, "files")) != NULL))
+			{
+				char_u		c;
+				char_u		fc = '=';
+
+				while (*q && (q = STRCHR(q, fc)) != NULL)
+				{
+					q++;
+					c = q[STRLEN(gettail(NameBuff))];
+# ifdef MSDOS
+					if ((c == NUL || c == ';' || c == ' ' || c == '\t')
+							&& vim_strnicmp(gettail(NameBuff), q, (size_t)STRLEN(gettail(NameBuff))) == 0)
+# else
+					if ((c == NUL || c == ';' || c == ' ' || c == '\t')
+										&& STRNCMP(gettail(NameBuff), q, STRLEN(gettail(NameBuff))) == 0)
+# endif
+					{
+						do_fexrc = TRUE;
+						break;
+					}
+					fc = ';';
+				}
+			}
+			else if (do_fexrc
+					&& (STRNCMP(p, "end", strlen("end")) == 0)
+					&& ((q = strstr(IObuff, "end")) != NULL)
+					&& ((q = strstr(q, "files")) != NULL))
+				do_fexrc = FALSE;
+			else if (do_fexrc)
+				memmove(IObuff, IObuff + 1, len);
 		}
-		else
 #endif
 		docmdline(IObuff);
 		len = 0;
@@ -3681,7 +3833,7 @@ get_address(ptr)
 						if (dosearch(c, cmd, FALSE, (long)1, FALSE, TRUE))
 							lnum = curwin->w_cursor.lnum;
 						curwin->w_cursor = pos;
-				
+
 						cmd += searchcmdlen;	/* adjust command string pointer */
 						break;
 
@@ -3689,7 +3841,7 @@ get_address(ptr)
 						if (isdigit(*cmd))				/* absolute line number */
 							lnum = getdigits(&cmd);
 		}
-		
+
 		while (*cmd == '-' || *cmd == '+')
 		{
 			if (lnum == MAXLNUM)
@@ -3697,7 +3849,7 @@ get_address(ptr)
 			i = *cmd++;
 			if (!isdigit(*cmd))	/* '+' is '+1', but '+0' is not '+1' */
 				n = 1;
-			else 
+			else
 				n = getdigits(&cmd);
 			if (i == '-')
 				lnum -= n;
@@ -3835,7 +3987,7 @@ set_one_cmd_context(firstc, buff)
 						++cmd;
 					break;
 			}
-			
+
 			while (*cmd == '-' || *cmd == '+')
 			{
 				cmd++;
@@ -4006,20 +4158,22 @@ set_one_cmd_context(firstc, buff)
 		{
 			int			j;
 			int			first = TRUE;
+
 			STRCPY(IObuff, arg);
 			p = &IObuff[strlen(IObuff)];
 			for (j = strlen(IObuff); j >= 0; j--)
 			{
-#ifdef KANJI
+# ifdef KANJI
 				if ((IObuff[j] == ' ' || IObuff[j] == TAB)
 								&& isdir(fileconvsto(&IObuff[j + 1])) == OK)
-#else
+# else
 				if ((IObuff[j] == ' ' || IObuff[j] == TAB)
 											&& isdir(&IObuff[j + 1]) == OK)
-#endif
+# endif
 				{
 					expand_pattern = arg + j + 1;
-					break;
+					if (!(argt & NOSPC))
+						break;
 				}
 				if ((IObuff[j] == '/' || IObuff[j] == '\\') && first == TRUE)
 				{
@@ -4027,13 +4181,13 @@ set_one_cmd_context(firstc, buff)
 					first = FALSE;
 				}
 			}
-#ifdef KANJI
+# ifdef KANJI
 			if (j <= 0 && (IObuff[0] != ' ' && IObuff[0] != TAB)
 										&& isdir(fileconvsto(&IObuff[0])) == OK)
-#else
+# else
 			if (j <= 0 && (IObuff[0] != ' ' && IObuff[0] != TAB)
 											&& isdir(&IObuff[0]) == OK)
-#endif
+# endif
 				expand_pattern = arg;
 		}
 	}
@@ -4241,7 +4395,7 @@ ExpandFromContext(pat, num_file, file, files_only, list_notfound)
 		ret = ExpandTags(prog, num_file, file);
 	else
 		ret = FAIL;
-	
+
 	free(prog);
 	return ret;
 }
