@@ -77,6 +77,10 @@ edit(count)
 	int			 done;
 	int			 found_error = FALSE;
 	char_u		 backup_char = 0;			/* init for gcc */
+# ifdef KANJI
+	static int	 ocls;
+	int			 scls;
+# endif
 
 	c = NUL;
 #endif /* WEBB_KEYWORD_COMPL */
@@ -109,10 +113,18 @@ edit(count)
 		o_eol = FALSE;
 #ifdef FEPCTRL
 		switch (*curbuf->b_p_ji) {
+		case 'j':
+			if (!p_ja)
+				break;
+			/* no break */
 		case 'J':
 			if (!fep_get_mode())
 				fep_force_on();
 			break;
+		case 'a':
+			if (!p_ja)
+				break;
+			/* no break */
 		case 'A':
 			if (fep_get_mode())
 				fep_force_off();
@@ -172,7 +184,7 @@ edit(count)
 				char		ji;
 
 				ji = *curbuf->b_p_ji;
-				if (ISkanji(c))
+				if (ISkanji(c) || ISkana(c))
 				{
 					if (curbuf->knj_asc > p_ja)
 						ji = 'J';
@@ -187,7 +199,7 @@ edit(count)
 						curbuf->knj_asc --;
 				}
 
-				if (ji != *curbuf->b_p_ji && isupper(*curbuf->b_p_ji))
+				if (ji != *curbuf->b_p_ji)
 				{
 					*curbuf->b_p_ji = ji;
 				}
@@ -561,7 +573,7 @@ dodel:
 										State = REPLACE;
 									}
 								}
-								else if (!p_jrep)
+								else if (p_jrep)
 								{
 									char *p, *c, *s;
 
@@ -592,7 +604,7 @@ dodel:
 										delchar(FALSE);
 									}
 								}
-								else /* p_jrep */
+								else /* !p_jrep */
 								{
 									char *p, *o, *s;
 									int  i;
@@ -803,7 +815,15 @@ redraw:
 
 			  case K_UARROW:
 			  	if (oneup(1L))
+#ifdef KANJI
+				{
+					if (ISkanjiCur() == 2)
+						curwin->w_cursor.col++;
 					start_arrow();
+				}
+#else
+					start_arrow();
+#endif
 				else
 					beep();
 				break;
@@ -817,7 +837,15 @@ redraw:
 
 			  case K_DARROW:
 			  	if (onedown(1L))
+#ifdef KANJI
+				{
+					if (ISkanjiCur() == 2)
+						curwin->w_cursor.col++;
 					start_arrow();
+				}
+#else
+					start_arrow();
+#endif
 				else
 					beep();
 				break;
@@ -944,6 +972,7 @@ redraw:
 					complete_col = complete_pos.col;
 					temp = complete_col - 1;
 #ifdef KANJI
+					ocls = jpcls('A', NUL);
 					if (temp < 0
 							|| !((ISkanjiPosition(ptr, temp + 1) == 2)
 												|| isidchar(ptr[temp])))
@@ -957,6 +986,10 @@ redraw:
 					else
 					{
 #ifdef KANJI
+						if (ISkanjiPosition(ptr, temp + 1) == 2)
+							ocls = jpcls(ptr[temp - 1], ptr[temp]);
+						else
+							ocls = jpcls(ptr[temp], ptr[temp + 1]);
 						while (temp >= 0)
 						{
 							if (ISkanjiPosition(ptr, temp + 1) == 2)
@@ -965,6 +998,12 @@ redraw:
 								temp --;
 							else
 								break;
+							if (ISkanjiPosition(ptr, temp + 1) == 2)
+								scls = jpcls(ptr[temp - 1], ptr[temp]);
+							else
+								scls = jpcls(ptr[temp], ptr[temp + 1]);
+							if (ocls != scls)
+								break;
 						}
 #else
 						while (temp >= 0 && isidchar(ptr[temp]))
@@ -1023,6 +1062,12 @@ redraw:
 								temp --;
 							else
 								break;
+							if (ISkanjiPosition(ptr, temp + 1) == 2)
+								scls = jpcls(ptr[temp - 1], ptr[temp]);
+							else
+								scls = jpcls(ptr[temp], ptr[temp + 1]);
+							if (ocls != scls)
+								break;
 						}
 						complete_pos.col = temp;
 #else
@@ -1046,6 +1091,9 @@ redraw:
 #ifdef KANJI
 					while (*tmp_ptr != NUL)
 					{
+						scls = jpcls(tmp_ptr[0], tmp_ptr[1]);
+						if (ocls != scls)
+							break;
 						if (ISkanji(*tmp_ptr))
 						{
 							temp += 2;
@@ -1073,6 +1121,9 @@ redraw:
 #ifdef KANJI
 					while (*ptr != NUL)
 					{
+						scls = jpcls(ptr[0], ptr[1]);
+						if (ocls != scls)
+							break;
 						if (ISkanji(*ptr))
 						{
 							*(tmp_ptr++) = *(ptr++);
@@ -1242,6 +1293,32 @@ get_literal(nextc)
 			*kp = vgetc();
 			break;
 		}
+		if (nc == '#')
+		{
+			for (i = 0; i < 4; ++i)
+			{
+				nc = vgetc();
+				if (ISkanji(nc))
+					*kp = vgetc();
+				if (!isxdigit(nc))
+				{
+					cc = '#';
+					i = 1;
+					break;
+				}
+				if ('0' <= nc && nc <= '9')
+					cc = cc * 16 + nc - '0';
+				else
+					cc = cc * 16 + toupper(nc) - 'A' + 10;
+			}
+			if (i >= 4)
+			{
+				nc  = (cc & 0xff00) >> 8;
+				*kp =  cc & 0x00ff;
+				i   = 0;
+			}
+			break;
+		}
 #endif
 		if (!isdigit(nc))
 			break;
@@ -1258,6 +1335,29 @@ get_literal(nextc)
 #ifdef KANJI
 	else if (cc >= 0x100)
 		cc &= 0x7f;
+	else if (ISkanji(cc))
+	{
+		*kp = 0;
+		for (i = 0; i < 3; ++i)
+		{
+			nc = vgetc();
+			if (ISkanji(nc))
+			{
+				*kp = vgetc();
+				cc = '\n';
+				nc = 0;
+				break;
+			}
+			if (!isdigit(nc))
+			{
+				cc = '\n';
+				nc = 0;
+				break;
+			}
+			*kp = *kp * 10 + nc - '0';
+			nc = 0;
+		}
+	}
 #endif
 	else if (cc == 0)		/* NUL is stored as NL */
 		cc = '\n';
@@ -1333,7 +1433,7 @@ insertchar(c)
 			if ((startcol = curwin->w_cursor.col) == 0)
 				break;
 #ifdef KANJI
-			curwin->w_cursor.col = vcol2col(curwin, curwin->w_cursor.lnum, textwidth, &wantcol);
+			curwin->w_cursor.col = vcol2col(curwin, curwin->w_cursor.lnum, textwidth, &wantcol, 0, 0);
 #else
 			coladvance(textwidth);			/* find column of textwidth border */
 			wantcol = curwin->w_cursor.col;
@@ -1386,8 +1486,8 @@ insertchar(c)
 #ifdef KANJI	/* KINSOKU syori */
 			if (foundcol == startcol)
 			{
-				if (!c || (ISkanji(c) ? isjppunc(c, k, TRUE) /* close punc. */
-									  : isaspunc(c, TRUE)) )
+				if (!c || (ISkanji(c) ? isjppunc((char_u)c, (char_u)k, TRUE)
+									  : isaspunc((char_u)c, TRUE)) )
 					break;
 			}
 			else if (foundcol < startcol)	/* for closing symbols */
