@@ -246,6 +246,18 @@ char		*	fname;
 	return(TRUE);
 }
 
+/*------------------------------------------------------------------------------
+ *	CHECK Cab file
+ *----------------------------------------------------------------------------*/
+BOOL
+cab_get_aname(fname, sep, aname)
+const char	*	fname;
+char			sep;
+char		*	aname;
+{
+	return(cab_getname(fname, sep, aname, NULL));
+}
+
 /*------------------------------------------------------------------------------
  *	open() emulation
  *----------------------------------------------------------------------------*/
