@@ -22,6 +22,9 @@
 /*==============================================================================
  *	EXtended FILE support routine
  *============================================================================*/
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
 typedef struct {
 	int				fd;						/* open dummy file descriptor	  */
 	int				mode;					/* open mode					  */
@@ -31,6 +34,17 @@ typedef struct {
 static TmpFile		Tempfile[16];
 static char			Tempbase[MAX_PATH];
 
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+typedef struct {
+	HANDLE			hFile;					/* open Share File handle		  */
+	int				fd;						/* file descriptor				  */
+	BOOL			permit;					/*								  */
+	char			filename[MAX_PATH];		/* original file name			  */
+} ShareFile;
+static ShareFile	Sharefile[32];
+
 typedef struct find_data {
 	struct find_data *	next;
 	WIN32_FIND_DATA		data;
@@ -45,6 +59,409 @@ static BOOL			ef_load = FALSE;
 BOOL				ef_dir = TRUE;
 static char			ef_version[4096];
 
+static BOOL			ef_share = FALSE;
+static DWORD		ef_share_busy_count = 0;
+
+/*==============================================================================
+ *	Shared FILE support routine
+ *============================================================================*/
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+static void
+ef_share_waitcursor(flg)
+BOOL			flg;
+{
+	static HCURSOR	hWait = NULL;
+	static HCURSOR	hOrig = NULL;
+
+	if (flg)
+	{
+		if (hWait == NULL)
+			hWait = LoadCursor(NULL, IDC_WAIT);
+		if (ef_share_busy_count == 0 && hWait != NULL)
+			hOrig = SetCursor(hWait);
+		ef_share_busy_count++;
+	}
+	else if (ef_share_busy_count)
+	{
+		ef_share_busy_count--;
+		if (ef_share_busy_count == 0)
+			SetCursor(hOrig);
+	}
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+static void
+ef_get_longpath(sname, lname)
+char		*	sname;
+char		*	lname;
+{
+	char		*	wk;
+
+	_fullpath(lname, sname, MAX_PATH);
+	if ((wk = malloc(strlen(lname) + 1)) != NULL)
+	{
+		strcpy(wk, lname);
+		wk = ef_getlongname(wk);
+		strcpy(lname, wk);
+		free(wk);
+	}
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+VOID
+ef_share_init()
+{
+	int						i;
+
+	if (!ef_share)
+	{
+		for (i = 0; i < (sizeof(Sharefile) / sizeof(ShareFile)); i++)
+		{
+			Sharefile[i].hFile			= INVALID_HANDLE_VALUE;
+			Sharefile[i].fd				= -1;
+			Sharefile[i].permit			= FALSE;
+			Sharefile[i].filename[0]	= '\0';
+		}
+	}
+	ef_share = TRUE;
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+VOID
+ef_share_term()
+{
+	int						i;
+
+	if (ef_share)
+	{
+		for (i = 0; i < (sizeof(Sharefile) / sizeof(ShareFile)); i++)
+		{
+			if (Sharefile[i].hFile != INVALID_HANDLE_VALUE)
+				CloseHandle(Sharefile[i].hFile);
+			Sharefile[i].hFile = INVALID_HANDLE_VALUE;
+		}
+	}
+	ef_share = FALSE;
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+BOOL
+ef_share_busy()
+{
+	if (!ef_share)
+		return(FALSE);
+	return(ef_share_busy_count);
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+BOOL
+ef_share_check(const char *fname, int mode)
+{
+	SECURITY_ATTRIBUTES		security;
+	HANDLE					handle;
+	char					aname[MAX_PATH];
+	char					target[MAX_PATH];
+
+	if (!ef_share)
+		return(FALSE);
+	if (is_ftp(fname, ':'))
+		return(FALSE);
+	if (is_ef(fname, ':'))
+	{
+		if (ef_get_aname(fname, ':', aname))
+			fname = aname;
+	}
+	ef_get_longpath(fname, target);
+	security.nLength = sizeof(security);
+	security.lpSecurityDescriptor = NULL;
+	security.bInheritHandle = FALSE;	/* we don't hope inherit... */
+	handle = CreateFile(target,
+			mode & (O_WRONLY | O_RDWR) ?
+									GENERIC_READ|GENERIC_WRITE : GENERIC_READ,
+			mode & (O_WRONLY | O_RDWR) ? 0 : FILE_SHARE_READ,
+			&security, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, (HANDLE)NULL);
+	if (handle == INVALID_HANDLE_VALUE)
+	{
+		switch (GetLastError()) {
+		case ERROR_SHARING_VIOLATION:
+			return(mode & (O_WRONLY | O_RDWR) ? TRUE : FALSE);
+		case ERROR_ACCESS_DENIED:
+			return(TRUE);
+		case ERROR_FILE_NOT_FOUND:
+			return(FALSE);
+		}
+	}
+	else
+		CloseHandle(handle);
+	return(FALSE);
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+BOOL
+ef_share_process(const char *fname)
+{
+	SECURITY_ATTRIBUTES		security;
+	HANDLE					handle;
+	char					aname[MAX_PATH];
+	char					target[MAX_PATH];
+	int						i;
+
+	if (!ef_share)
+		return(FALSE);
+	if (fname == NULL)
+		return(FALSE);
+	if (is_ftp(fname, ':'))
+		return(FALSE);
+	if (is_ef(fname, ':'))
+	{
+		if (ef_get_aname(fname, ':', aname))
+			fname = aname;
+	}
+	ef_get_longpath(fname, target);
+	for (i = 0; i < (sizeof(Sharefile) / sizeof(ShareFile)); i++)
+	{
+		if (Sharefile[i].hFile != INVALID_HANDLE_VALUE
+								&& stricmp(Sharefile[i].filename, target) == 0)
+			return(FALSE);
+	}
+	security.nLength = sizeof(security);
+	security.lpSecurityDescriptor = NULL;
+	security.bInheritHandle = FALSE;	/* we don't hope inherit... */
+	handle = CreateFile(target, GENERIC_READ, 0,
+				&security, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, (HANDLE)NULL);
+	if (handle == INVALID_HANDLE_VALUE)
+	{
+		switch (GetLastError()) {
+		case ERROR_SHARING_VIOLATION:
+			return(TRUE);
+		default:
+			return(FALSE);
+		}
+	}
+	CloseHandle(handle);
+	return(FALSE);
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+VOID
+ef_share_open(const char *fname)
+{
+	SECURITY_ATTRIBUTES		security;
+	HANDLE					handle;
+	char					aname[MAX_PATH];
+	char					target[MAX_PATH];
+	int						i;
+
+	if (!ef_share)
+		return;
+	if (fname == NULL)
+		return;
+	if (is_ftp(fname, ':'))
+		return;
+	if (is_ef(fname, ':'))
+	{
+		if (ef_get_aname(fname, ':', aname))
+			fname = aname;
+	}
+	ef_get_longpath(fname, target);
+	for (i = 0; i < (sizeof(Sharefile) / sizeof(ShareFile)); i++)
+	{
+		if (Sharefile[i].hFile != INVALID_HANDLE_VALUE
+								&& stricmp(Sharefile[i].filename, target) == 0)
+			return;
+	}
+	for (i = 0; i < (sizeof(Sharefile) / sizeof(ShareFile)); i++)
+	{
+		if (Sharefile[i].hFile == INVALID_HANDLE_VALUE)
+			break;
+	}
+	if (i >= (sizeof(Sharefile) / sizeof(ShareFile)))
+		return;
+	security.nLength = sizeof(security);
+	security.lpSecurityDescriptor = NULL;
+	security.bInheritHandle = FALSE;	/* we don't hope inherit... */
+	handle = CreateFile(target, GENERIC_READ, 0,
+				&security, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, (HANDLE)NULL);
+	if (handle == INVALID_HANDLE_VALUE)
+	{
+		switch (GetLastError()) {
+		case ERROR_FILE_NOT_FOUND:
+			handle = CreateFile(target, GENERIC_READ|GENERIC_WRITE, 0,
+					&security, CREATE_NEW,
+					FILE_ATTRIBUTE_NORMAL|FILE_FLAG_DELETE_ON_CLOSE,
+					(HANDLE)NULL);
+			break;
+		}
+	}
+	else
+	{
+		CloseHandle(handle);
+		handle = CreateFile(target, GENERIC_READ, FILE_SHARE_READ,
+				&security, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, (HANDLE)NULL);
+	}
+	Sharefile[i].hFile = handle;
+	strcpy(Sharefile[i].filename, target);
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+VOID
+ef_share_close(const char *fname)
+{
+	char					aname[MAX_PATH];
+	char					target[MAX_PATH];
+	int						i;
+
+	if (!ef_share)
+		return;
+	if (fname == NULL)
+		return;
+	if (is_ftp(fname, ':'))
+		return;
+	if (is_ef(fname, ':'))
+	{
+		if (ef_get_aname(fname, ':', aname))
+			fname = aname;
+	}
+	ef_get_longpath(fname, target);
+	for (i = 0; i < (sizeof(Sharefile) / sizeof(ShareFile)); i++)
+	{
+		if (stricmp(Sharefile[i].filename, target) == 0)
+		{
+			if (Sharefile[i].hFile != INVALID_HANDLE_VALUE)
+				CloseHandle(Sharefile[i].hFile);
+			Sharefile[i].hFile			= INVALID_HANDLE_VALUE;
+			Sharefile[i].fd				= -1;
+			Sharefile[i].permit			= FALSE;
+			Sharefile[i].filename[0]	= '\0';
+			return;
+		}
+	}
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+static int
+ef_share_opened(const char *fname)
+{
+	char					aname[MAX_PATH];
+	char					target[MAX_PATH];
+	int						i;
+
+	if (!ef_share)
+		return(-1);
+	if (is_ftp(fname, ':'))
+		return(-1);
+	if (is_ef(fname, ':'))
+	{
+		if (ef_get_aname(fname, ':', aname))
+			fname = aname;
+	}
+	ef_get_longpath(fname, target);
+	for (i = 0; i < (sizeof(Sharefile) / sizeof(ShareFile)); i++)
+	{
+		if (Sharefile[i].hFile != INVALID_HANDLE_VALUE
+								&& Sharefile[i].permit != TRUE
+								&& stricmp(Sharefile[i].filename, target) == 0)
+		{
+			if (Sharefile[i].hFile != INVALID_HANDLE_VALUE)
+				CloseHandle(Sharefile[i].hFile);
+			Sharefile[i].permit	= TRUE;
+			ef_share_waitcursor(TRUE);
+			return(i);
+		}
+	}
+	return(-1);
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+static VOID
+ef_share_permit(int array, int fd)
+{
+	if (!ef_share)
+		return;
+	if (array < 0)
+		return;
+	if (fd < 0)
+		return;
+	Sharefile[array].fd = fd;
+}
+
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+static VOID
+ef_share_prohibit(int array, int fd)
+{
+	SECURITY_ATTRIBUTES		security;
+	HANDLE					handle;
+	int						i;
+
+	if (!ef_share)
+		return;
+	if (array < 0 && fd < 0)
+		return;
+	if (array >= 0 && fd >= 0)
+		return;
+	if (fd >= 0)
+	{
+		for (i = 0; i < (sizeof(Sharefile) / sizeof(ShareFile)); i++)
+		{
+			if (Sharefile[i].fd == fd)
+			{
+				array = i;
+				break;
+			}
+		}
+	}
+	if (array < 0)
+		return;
+	if (!Sharefile[array].permit)
+		return;
+	security.nLength = sizeof(security);
+	security.lpSecurityDescriptor = NULL;
+	security.bInheritHandle = FALSE;
+	handle = CreateFile(Sharefile[array].filename,
+				GENERIC_READ, FILE_SHARE_READ, &security,
+				OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, (HANDLE)NULL);
+	if (handle == INVALID_HANDLE_VALUE)
+	{
+		switch (GetLastError()) {
+		case ERROR_FILE_NOT_FOUND:
+			handle = CreateFile(Sharefile[array].filename,
+						GENERIC_READ|GENERIC_WRITE, 0,
+						&security, CREATE_NEW,
+						FILE_ATTRIBUTE_NORMAL|FILE_FLAG_DELETE_ON_CLOSE,
+						(HANDLE)NULL);
+			break;
+		}
+	}
+	Sharefile[array].hFile	= handle;
+	Sharefile[array].fd		= -1;
+	Sharefile[array].permit	= FALSE;
+	ef_share_waitcursor(FALSE);
+}
+
 /*==============================================================================
  *	EXtended FILE support routine
  *============================================================================*/
@@ -293,6 +710,30 @@ char		*	fname;
 	return(FALSE);
 }
 
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+BOOL
+ef_get_aname(fname, sep, aname)
+const char	*	fname;
+char			sep;
+char		*	aname;
+{
+	if (!ef_load)
+		return(FALSE);
+	if (lha_get_aname(fname, sep, aname))
+		return(TRUE);
+	if (tar_get_aname(fname, sep, aname))
+		return(TRUE);
+	if (gzip_get_aname(fname, sep, aname))
+		return(TRUE);
+	if (zip_get_aname(fname, sep, aname))
+		return(TRUE);
+	if (cab_get_aname(fname, sep, aname))
+		return(TRUE);
+	return(FALSE);
+}
+
 /*------------------------------------------------------------------------------
  *	EXtended FILE initialize
  *----------------------------------------------------------------------------*/
@@ -326,42 +767,49 @@ HWND			window;
 		strcat(Tempbase, "efXXXXXX");
 		w = SetErrorMode(SEM_NOOPENFILEERRORBOX);
 		ef_version[0] = '\0';
+		ef_loop();
 		if (lha_init(window))
 		{
 			for (i = 0; i < VER_LEFT; i++)
 				strcat(ef_version, " ");
 			sprintf(&ef_version[strlen(ef_version)], "%s\r\n", lha_ver());
 		}
+		ef_loop();
 		if (ftp_init(window))
 		{
 			for (i = 0; i < VER_LEFT; i++)
 				strcat(ef_version, " ");
 			sprintf(&ef_version[strlen(ef_version)], "%s\r\n", ftp_ver());
 		}
+		ef_loop();
 		if (tar_init(window))
 		{
 			for (i = 0; i < VER_LEFT; i++)
 				strcat(ef_version, " ");
 			sprintf(&ef_version[strlen(ef_version)], "%s\r\n", tar_ver());
 		}
+		ef_loop();
 		if (gzip_init(window))
 		{
 			for (i = 0; i < VER_LEFT; i++)
 				strcat(ef_version, " ");
 			sprintf(&ef_version[strlen(ef_version)], "%s\r\n", gzip_ver());
 		}
+		ef_loop();
 		if (zip_init(window))
 		{
 			for (i = 0; i < VER_LEFT; i++)
 				strcat(ef_version, " ");
 			sprintf(&ef_version[strlen(ef_version)], "%s\r\n", zip_ver());
 		}
+		ef_loop();
 		if (cab_init(window))
 		{
 			for (i = 0; i < VER_LEFT; i++)
 				strcat(ef_version, " ");
 			sprintf(&ef_version[strlen(ef_version)], "%s", cab_ver());
 		}
+		ef_loop();
 		SetErrorMode(w);
 		ef_load = TRUE;
 	}
@@ -457,84 +905,108 @@ char			sep;
  *	EXtended FILE open
  *----------------------------------------------------------------------------*/
 int
-ef_open(fname, mode, ...)
-const char	*	fname;
-int				mode;
+ef_open(const char *fname, int mode, ...)
 {
     va_list				ap;
 	WIN32_FIND_DATA		data;
 	BOOL				newfile	= TRUE;
 	int					i;
-	int					rtn;
+	int					rtn = 0;
 	TmpFile			*	tmp	= NULL;
 	HANDLE				hFile;
     int					pmode;
+	int					share;
 
-    va_start(ap, mode);
-    pmode = va_arg(ap, int);
-    va_end(ap);
-	if (!ef_load || !is_ef(fname, ':'))
-		return(open(fname, mode, pmode));
-	if (ef_isdir(fname))
-		return(-1);
-	if (strchr(fname, '*') != NULL || strchr(fname, '?') != NULL)
-		return(-1);
-	if ((hFile = ef_findfirst(fname, &data)) == INVALID_HANDLE_VALUE)
+	va_start(ap, mode);
+	pmode = va_arg(ap, int);
+	va_end(ap);
+	share = ef_share_opened(fname);
+	if (ef_share_check(fname, mode))
+		rtn = (-1);
+	else if (!ef_load || !is_ef(fname, ':'))
+		rtn = open(fname, mode, pmode);
+	else if (ef_isdir(fname))
+		rtn = (-1);
+	else if (strchr(fname, '*') != NULL || strchr(fname, '?') != NULL)
+		rtn = (-1);
+	else if ((hFile = ef_findfirst(fname, &data)) == INVALID_HANDLE_VALUE)
 	{
 		if ((mode & (O_WRONLY | O_CREAT)) && !ef_sfx(fname))
 			goto newarch;
-		return(-1);
+		rtn = (-1);
 	}
-	ef_findclose(hFile);
-	if (((data.dwFileAttributes & FILE_ATTRIBUTE_READONLY) || ef_sfx(fname))
-												&& (mode & (O_WRONLY | O_RDWR)))
-		return(-1);
-	newfile = FALSE;
-newarch:
-	for (i = 0; i < (sizeof(Tempfile) / sizeof(TmpFile)); i++)
-	{
-		if (Tempfile[i].fd == (-1))
-		{
-			strcpy(Tempfile[i].tempname, Tempbase);
-			if (mktemp(Tempfile[i].tempname) == NULL)
-				return(-1);
-			strcpy(Tempfile[i].filename, fname);
-			tmp = &Tempfile[i];
-			break;
-		}
-	}
-	if (tmp == NULL)
-		return(-1);
-	if (newfile)
-		;
 	else
 	{
-		if (is_lha(fname, ':'))
-			rtn = lha_getfile(fname, tmp->tempname);
-		else if (is_ftp(fname, ':'))
-			rtn = ftp_getfile(fname, tmp->tempname);
-		else if (is_tar(fname, ':'))
-			rtn = tar_getfile(fname, tmp->tempname);
-		else if (is_gzip(fname, ':'))
-			rtn = gzip_getfile(fname, tmp->tempname);
-		else if (is_zip(fname, ':'))
-			rtn = zip_getfile(fname, tmp->tempname);
-		else if (is_cab(fname, ':'))
-			rtn = cab_getfile(fname, tmp->tempname);
-		chmod(tmp->tempname, S_IREAD | S_IWRITE);
-		if (!rtn)
+		ef_findclose(hFile);
+		if (((data.dwFileAttributes & FILE_ATTRIBUTE_READONLY) || ef_sfx(fname))
+												&& (mode & (O_WRONLY | O_RDWR)))
+			rtn = (-1);
+		else
 		{
-			unlink(tmp->tempname);
-			return(-1);
+			newfile = FALSE;
+newarch:
+			for (i = 0; i < (sizeof(Tempfile) / sizeof(TmpFile)); i++)
+			{
+				if (Tempfile[i].fd == (-1))
+				{
+					strcpy(Tempfile[i].tempname, Tempbase);
+					if (mktemp(Tempfile[i].tempname) == NULL)
+						rtn = (-1);
+					else
+					{
+						strcpy(Tempfile[i].filename, fname);
+						tmp = &Tempfile[i];
+					}
+					break;
+				}
+			}
+			if (tmp == NULL)
+				rtn = (-1);
+			else if (rtn != 0 || newfile)
+				;
+			else
+			{
+				if (is_lha(fname, ':'))
+					rtn = lha_getfile(fname, tmp->tempname);
+				else if (is_ftp(fname, ':'))
+					rtn = ftp_getfile(fname, tmp->tempname);
+				else if (is_tar(fname, ':'))
+					rtn = tar_getfile(fname, tmp->tempname);
+				else if (is_gzip(fname, ':'))
+					rtn = gzip_getfile(fname, tmp->tempname);
+				else if (is_zip(fname, ':'))
+					rtn = zip_getfile(fname, tmp->tempname);
+				else if (is_cab(fname, ':'))
+					rtn = cab_getfile(fname, tmp->tempname);
+				chmod(tmp->tempname, S_IREAD | S_IWRITE);
+				if (!rtn)
+				{
+					unlink(tmp->tempname);
+					rtn = (-1);
+				}
+				else
+				{
+					if ((rtn = open(tmp->tempname, mode, S_IREAD|S_IWRITE)) < 0)
+					{
+						unlink(tmp->tempname);
+						rtn = (-1);
+					}
+					else
+					{
+						tmp->fd = rtn;
+						tmp->mode = mode;
+					}
+				}
+			}
 		}
 	}
-	if ((rtn = open(tmp->tempname, mode, S_IREAD | S_IWRITE)) < 0)
+	if (share >= 0)
 	{
-		unlink(tmp->tempname);
-		return(-1);
+		if (rtn >= 0)
+			ef_share_permit(share, rtn);
+		else
+			ef_share_prohibit(share, -1);
 	}
-	tmp->fd = rtn;
-	tmp->mode = mode;
 	return(rtn);
 }
 
@@ -546,58 +1018,70 @@ ef_close(fd)
 int				fd;
 {
 	int					i;
+	int					rtn;
 	BOOL				change = FALSE;
 
+	rtn = close(fd);
 	if (!ef_load)
-		return(close(fd));
-	if (close(fd) != 0)
-		return(-1);
-	for (i = 0; i < (sizeof(Tempfile) / sizeof(TmpFile)); i++)
+		;
+	else if (rtn != 0)
+		rtn = (-1);
+	else
 	{
-		if (Tempfile[i].fd == fd)
+		for (i = 0; i < (sizeof(Tempfile) / sizeof(TmpFile)); i++)
 		{
-			if (Tempfile[i].mode & (O_WRONLY | O_CREAT))
-				change = TRUE;
-			if (change)
+			if (Tempfile[i].fd == fd)
 			{
-				if (is_lha(Tempfile[i].filename, ':'))
+				if (Tempfile[i].mode & (O_WRONLY | O_CREAT))
+					change = TRUE;
+				if (change)
 				{
-					if (!lha_putfile(Tempfile[i].filename, Tempfile[i].tempname))
-						return(-1);
-				}
-				else if (is_ftp(Tempfile[i].filename, ':'))
-				{
-					if (!ftp_putfile(Tempfile[i].filename, Tempfile[i].tempname))
-						return(-1);
-				}
-				else if (is_tar(Tempfile[i].filename, ':'))
-				{
-					if (!tar_putfile(Tempfile[i].filename, Tempfile[i].tempname))
-						return(-1);
-				}
-				else if (is_gzip(Tempfile[i].filename, ':'))
-				{
-					if (!gzip_putfile(Tempfile[i].filename, Tempfile[i].tempname))
-						return(-1);
-				}
-				else if (is_zip(Tempfile[i].filename, ':'))
-				{
-					if (!zip_putfile(Tempfile[i].filename, Tempfile[i].tempname))
-						return(-1);
-				}
-				else if (is_cab(Tempfile[i].filename, ':'))
-				{
-					if (!cab_putfile(Tempfile[i].filename, Tempfile[i].tempname))
-						return(-1);
+					if (is_lha(Tempfile[i].filename, ':'))
+					{
+						if (!lha_putfile(Tempfile[i].filename,
+														Tempfile[i].tempname))
+							rtn = (-1);
+					}
+					else if (is_ftp(Tempfile[i].filename, ':'))
+					{
+						if (!ftp_putfile(Tempfile[i].filename,
+														Tempfile[i].tempname))
+							rtn = (-1);
+					}
+					else if (is_tar(Tempfile[i].filename, ':'))
+					{
+						if (!tar_putfile(Tempfile[i].filename,
+														Tempfile[i].tempname))
+							rtn = (-1);
+					}
+					else if (is_gzip(Tempfile[i].filename, ':'))
+					{
+						if (!gzip_putfile(Tempfile[i].filename,
+														Tempfile[i].tempname))
+							rtn = (-1);
+					}
+					else if (is_zip(Tempfile[i].filename, ':'))
+					{
+						if (!zip_putfile(Tempfile[i].filename,
+														Tempfile[i].tempname))
+							rtn = (-1);
+					}
+					else if (is_cab(Tempfile[i].filename, ':'))
+					{
+						if (!cab_putfile(Tempfile[i].filename,
+														Tempfile[i].tempname))
+							rtn = (-1);
+					}
 				}
+				chmod(Tempfile[i].tempname, S_IREAD | S_IWRITE);
+				unlink(Tempfile[i].tempname);
+				Tempfile[i].fd = -1;
+				break;
 			}
-			chmod(Tempfile[i].tempname, S_IREAD | S_IWRITE);
-			unlink(Tempfile[i].tempname);
-			Tempfile[i].fd = -1;
-			break;
 		}
 	}
-	return(0);
+	ef_share_prohibit(-1, fd);
+	return(rtn);
 }
 
 /*------------------------------------------------------------------------------
@@ -607,16 +1091,18 @@ int
 ef_remove(fname)
 const char	*	fname;
 {
-	int					rtn;
+	int					rtn = -1;
+	int					share;
 
+	share = ef_share_opened(fname);
 	if (!ef_load || !is_ef(fname, ':'))
 	{
 		chmod(fname, S_IREAD | S_IWRITE);
-		return(unlink(fname));
+		rtn = unlink(fname);
 	}
-	if (ef_sfx(fname))
-		return(-1);
-	if (is_lha(fname, ':'))
+	else if (ef_sfx(fname))
+		rtn = -1;
+	else if (is_lha(fname, ':'))
 		rtn = lha_remove(fname);
 	else if (is_ftp(fname, ':'))
 		rtn = ftp_remove(fname);
@@ -628,6 +1114,7 @@ const char	*	fname;
 		rtn = zip_remove(fname);
 	else if (is_cab(fname, ':'))
 		rtn = cab_remove(fname);
+	ef_share_prohibit(share, -1);
 	return(rtn);
 }
 
@@ -639,79 +1126,91 @@ ef_rename(old, new)
 const char	*	old;
 const char	*	new;
 {
-	int					rtn;
+	int					rtn = 0;
 	char				tmpname[MAX_PATH];
+	int					oshare;
+	int					nshare;
 
+	oshare = ef_share_opened(old);
+	nshare = ef_share_opened(new);
 	if (!ef_load || (!is_ef(old, ':') && !is_ef(new, ':')))
-		return(rename(old, new));
-	if (ef_sfx(old) || ef_sfx(new))
-		return(-1);
-	if (is_ef(old, ':'))
-	{
-		strcpy(tmpname, Tempbase);
-		if (mktemp(tmpname) == NULL)
-			return(-1);
-		if (is_lha(old, ':'))
-		{
-			if (!lha_getfile(old, tmpname))
-				return(-1);
-		}
-		else if (is_ftp(old, ':'))
-		{
-			if (!ftp_getfile(old, tmpname))
-				return(-1);
-		}
-		else if (is_tar(old, ':'))
-		{
-			if (!tar_getfile(old, tmpname))
-				return(-1);
-		}
-		else if (is_gzip(old, ':'))
-		{
-			if (!gzip_getfile(old, tmpname))
-				return(-1);
-		}
-		else if (is_zip(old, ':'))
-		{
-			if (!zip_getfile(old, tmpname))
-				return(-1);
-		}
-		else if (is_cab(old, ':'))
-		{
-			if (!cab_getfile(old, tmpname))
-				return(-1);
-		}
-	}
+		rtn = rename(old, new);
+	else if (ef_sfx(old) || ef_sfx(new))
+		rtn = (-1);
 	else
-		strcpy(tmpname, old);
-	if (is_ef(new, ':'))
-	{
-		if (is_lha(new, ':'))
-			rtn = lha_putfile(new, tmpname);
-		else if (is_ftp(new, ':'))
-			rtn = ftp_putfile(new, tmpname);
-		else if (is_tar(new, ':'))
-			rtn = tar_putfile(new, tmpname);
-		else if (is_gzip(new, ':'))
-			rtn = gzip_putfile(new, tmpname);
-		else if (is_zip(new, ':'))
-			rtn = zip_putfile(new, tmpname);
-		else if (is_cab(new, ':'))
-			rtn = cab_putfile(new, tmpname);
-		if (rtn)
-			rtn = 0;
-		else
-			rtn = -1;
-	}
-	else
-		rtn = rename(tmpname, new);
-	if (rtn != 0)
 	{
 		if (is_ef(old, ':'))
-			ef_remove(old);
-		return(-1);
+		{
+			strcpy(tmpname, Tempbase);
+			if (mktemp(tmpname) == NULL)
+				rtn = (-1);
+			else if (is_lha(old, ':'))
+			{
+				if (!lha_getfile(old, tmpname))
+					rtn = (-1);
+			}
+			else if (is_ftp(old, ':'))
+			{
+				if (!ftp_getfile(old, tmpname))
+					rtn = (-1);
+			}
+			else if (is_tar(old, ':'))
+			{
+				if (!tar_getfile(old, tmpname))
+					rtn = (-1);
+			}
+			else if (is_gzip(old, ':'))
+			{
+				if (!gzip_getfile(old, tmpname))
+					rtn = (-1);
+			}
+			else if (is_zip(old, ':'))
+			{
+				if (!zip_getfile(old, tmpname))
+					rtn = (-1);
+			}
+			else if (is_cab(old, ':'))
+			{
+				if (!cab_getfile(old, tmpname))
+					rtn = (-1);
+			}
+		}
+		else
+			strcpy(tmpname, old);
+		if (rtn == 0)
+		{
+			if (is_ef(new, ':'))
+			{
+				if (is_lha(new, ':'))
+					rtn = lha_putfile(new, tmpname);
+				else if (is_ftp(new, ':'))
+					rtn = ftp_putfile(new, tmpname);
+				else if (is_tar(new, ':'))
+					rtn = tar_putfile(new, tmpname);
+				else if (is_gzip(new, ':'))
+					rtn = gzip_putfile(new, tmpname);
+				else if (is_zip(new, ':'))
+					rtn = zip_putfile(new, tmpname);
+				else if (is_cab(new, ':'))
+					rtn = cab_putfile(new, tmpname);
+				if (rtn)
+					rtn = 0;
+				else
+					rtn = -1;
+			}
+			else
+				rtn = rename(tmpname, new);
+			if (rtn != 0)
+			{
+				if (is_ef(old, ':'))
+					ef_remove(old);
+				rtn = (-1);
+			}
+		}
 	}
-	return(0);
+	ef_share_prohibit(nshare, -1);
+	ef_share_prohibit(oshare, -1);
+	return(rtn);
 }
 
 /*------------------------------------------------------------------------------
@@ -725,34 +1224,41 @@ struct stat	*	st;
 	HANDLE				hFile;
 	WIN32_FIND_DATA		data;
 	unsigned short		r	= (-1);
+	long				rtn = 0;
+	int					share;
 
+	share = ef_share_opened(name);
 	if (!ef_load || !is_ef(name, ':'))
-		return(stat(name, st));
-	if (name[strlen(name) - 1] == '/' || name[strlen(name) - 1] == '\\')
+		rtn = stat(name, st);
+	else if (name[strlen(name) - 1] == '/' || name[strlen(name) - 1] == '\\')
 	{
 		st->st_mode = S_IREAD | S_IWRITE | S_IFDIR;
-		return(0);
+		rtn = (0);
 	}
-	if ((hFile = ef_findfirst(name, &data)) == INVALID_HANDLE_VALUE)
-		return(-1);
-	ef_findclose(hFile);
-	r = S_IREAD|S_IWRITE;
-	if (data.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
-		r &= ~S_IWRITE;
-	if (data.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN)
-		;
-	if (data.dwFileAttributes & FILE_ATTRIBUTE_SYSTEM)
-		;
-	if (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
-		r |= S_IFDIR;
-	if (data.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE)
-		;
-	if (data.dwFileAttributes & FILE_ATTRIBUTE_NORMAL)
-		;
-	if (data.dwFileAttributes & FILE_ATTRIBUTE_TEMPORARY)
-		;
-	st->st_mode = r;
-	return(0);
+	else if ((hFile = ef_findfirst(name, &data)) == INVALID_HANDLE_VALUE)
+		rtn = (-1);
+	else
+	{
+		ef_findclose(hFile);
+		r = S_IREAD|S_IWRITE;
+		if (data.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
+			r &= ~S_IWRITE;
+		if (data.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN)
+			;
+		if (data.dwFileAttributes & FILE_ATTRIBUTE_SYSTEM)
+			;
+		if (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+			r |= S_IFDIR;
+		if (data.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE)
+			;
+		if (data.dwFileAttributes & FILE_ATTRIBUTE_NORMAL)
+			;
+		if (data.dwFileAttributes & FILE_ATTRIBUTE_TEMPORARY)
+			;
+		st->st_mode = r;
+	}
+	ef_share_prohibit(share, -1);
+	return(rtn);
 }
 
 /*------------------------------------------------------------------------------
@@ -763,23 +1269,28 @@ ef_chmod(name, perm)
 const char	*	name;
 int				perm;
 {
+	int					rtn = -1;
+	int					share;
+
+	share = ef_share_opened(name);
 	if (!ef_load || !is_ef(name, ':'))
-		return(chmod(name, perm));
-	if (ef_sfx(name))
-		return(-1);
-	if (is_lha(name, ':'))
-		return(lha_chmod(name, perm));
-	if (is_ftp(name, ':'))
-		return(ftp_chmod(name, perm));
-	if (is_tar(name, ':'))
-		return(tar_chmod(name, perm));
-	if (is_gzip(name, ':'))
-		return(gzip_chmod(name, perm));
-	if (is_zip(name, ':'))
-		return(zip_chmod(name, perm));
-	if (is_cab(name, ':'))
-		return(cab_chmod(name, perm));
-	return(-1);
+		rtn = chmod(name, perm);
+	else if (ef_sfx(name))
+		rtn = -1;
+	else if (is_lha(name, ':'))
+		rtn = lha_chmod(name, perm);
+	else if (is_ftp(name, ':'))
+		rtn = ftp_chmod(name, perm);
+	else if (is_tar(name, ':'))
+		rtn = tar_chmod(name, perm);
+	else if (is_gzip(name, ':'))
+		rtn = gzip_chmod(name, perm);
+	else if (is_zip(name, ':'))
+		rtn = zip_chmod(name, perm);
+	else if (is_cab(name, ':'))
+		rtn = cab_chmod(name, perm);
+	ef_share_prohibit(share, -1);
+	return(rtn);
 }
 
 /*------------------------------------------------------------------------------
