@@ -236,6 +236,18 @@ char		*	fname;
 	return(FALSE);
 }
 
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+BOOL
+lha_get_aname(fname, sep, aname)
+const char	*	fname;
+char			sep;
+char		*	aname;
+{
+	return(lha_getname(fname, sep, aname, NULL));
+}
+
 /*------------------------------------------------------------------------------
  *	open() emulation
  *----------------------------------------------------------------------------*/
