@@ -68,6 +68,7 @@ static	char			*	tar_ext[]
 static	char			*	gzip_ext[]	= {".gz", ".Z", ".bz2", NULL};
 
 static	HWND				tar_disp;
+static	WORD				tar_version;
 #define LIBNAME				"tar32.dll"
 #define	CDISP				(16*1024)
 
@@ -114,7 +115,8 @@ HWND			window;
 			hTarLib = NULL;
 			return(FALSE);
 		}
-		if ((*pTarGetVersion)() < 23)
+		tar_version = (*pTarGetVersion)();
+		if (tar_version < 23)
 		{
 			FreeLibrary(hTarLib);
 			hTarLib = NULL;
@@ -249,6 +251,18 @@ char		*	fname;
 	return(TRUE);
 }
 
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+BOOL
+tar_get_aname(fname, sep, aname)
+const char	*	fname;
+char			sep;
+char		*	aname;
+{
+	return(tar_getname(fname, sep, aname, NULL));
+}
+
 /*------------------------------------------------------------------------------
  *	open() emulation
  *----------------------------------------------------------------------------*/
@@ -263,8 +277,8 @@ const char	*	tmp;
 	HANDLE				hFile;
 	WIN32_FIND_DATA		data;
 	int					fd;
-	char			*	p		= NULL;
-	char			*	w		= NULL;
+	char			*	p;
+	char			*	w;
 	int					i;
 
 	if (!tar_getname(fname, ':', tar_aname, tar_fname))
@@ -309,7 +323,7 @@ const char	*	tmp;
 	char				tar_aname[MAX_PATH];
 	char				tar_cline[MAX_PATH + 32];
 	struct stat			st;
-	char			*	p		= NULL;
+	char			*	p;
 	long 				perm	= -1;
 	int					fd;
 
@@ -886,6 +900,18 @@ char		*	fname;
 	return(TRUE);
 }
 
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+BOOL
+gzip_get_aname(fname, sep, aname)
+const char	*	fname;
+char			sep;
+char		*	aname;
+{
+	return(gzip_getname(fname, sep, aname, NULL));
+}
+
 /*------------------------------------------------------------------------------
  *	open() emulation
  *----------------------------------------------------------------------------*/
@@ -899,9 +925,39 @@ const char	*	tmp;
 
 	if (!gzip_getname(fname, ':', gzip_aname))
 		return(FALSE);
-	sprintf(gzip_cline, "--display-dialog=0 -xvfG \"%s\" \"%s\"", gzip_aname, tmp);
-	if ((*pTar)(tar_disp, gzip_cline, NULL, 0) != 0)
-		return(FALSE);
+	if (tar_version < 200)
+	{
+		sprintf(gzip_cline, "--display-dialog=0 -xvfG \"%s\" \"%s\"", gzip_aname, tmp);
+		if ((*pTar)(tar_disp, gzip_cline, NULL, 0) != 0)
+			return(FALSE);
+	}
+	else
+	{
+		HANDLE				hFile;
+		WIN32_FIND_DATA		data;
+		int					fd;
+		char			*	p;
+
+		if ((hFile = gzip_findfirst(fname, &data)) == INVALID_HANDLE_VALUE)
+			return(FALSE);
+		gzip_findclose(hFile);
+		if ((p = malloc(data.nFileSizeLow + 1000)) == NULL)
+			return(FALSE);
+		sprintf(gzip_cline, "--display-dialog=0 -pxvfG \"%s\"", gzip_aname);
+		if ((*pTar)(tar_disp, gzip_cline, p, data.nFileSizeLow + 1000) != 0)
+		{
+			free(p);
+			return(FALSE);
+		}
+		if ((fd = open(tmp, O_WRONLY | O_CREAT | O_BINARY, S_IREAD | S_IWRITE)) < 0)
+		{
+			free(p);
+			return(FALSE);
+		}
+		write(fd, p, data.nFileSizeLow);
+		close(fd);
+		free(p);
+	}
 	return(TRUE);
 }
 
@@ -1032,6 +1088,22 @@ WIN32_FIND_DATA	* 	data;
 	FindClose(hFind);
 	memset(data->cFileName, '\0', sizeof(data->cFileName));
 	gzip_makname(gzip_aname, data->cFileName);
+	if (tar_version >= 200)
+	{
+		HARC			hArc;
+
+		if ((*pTarCheckArchive)(gzip_aname, 1) == FALSE)
+			return(INVALID_HANDLE_VALUE);
+		if ((hArc = (*pTarOpenArchive)(tar_disp, gzip_aname, M_INIT_FILE_USE)) == NULL)
+			return(INVALID_HANDLE_VALUE);
+		if ((*pTarFindFirst)(hArc, "*.*", &SubInfo) == 0)
+		{
+			data->dwFileAttributes = FILE_ATTRIBUTE_NORMAL;
+			data->nFileSizeHigh = 0;
+			data->nFileSizeLow  = SubInfo.dwOriginalSize;
+		}
+		(*pTarCloseArchive)(hArc);
+	}
 	return(NULL);
 }
 
