@@ -8,6 +8,7 @@ VOID		tar_term(void);
 char	*	tar_ver(void);
 BOOL		is_tar(const char *, char);
 BOOL		tar_sfx(char *);
+BOOL		tar_get_aname(const char *, char, char *);
 BOOL		tar_getfile(const char *, const char *);
 BOOL		tar_putfile(const char *, const char *);
 int			tar_remove(const char *);
@@ -25,6 +26,7 @@ char	*	gzip_ver(void);
 char	*	gzip_version(void);
 BOOL		is_gzip(const char *, char);
 BOOL		gzip_sfx(char *);
+BOOL		gzip_get_aname(const char *, char, char *);
 BOOL		gzip_getfile(const char *, const char *);
 BOOL		gzip_putfile(const char *, const char *);
 int			gzip_remove(const char *);
