@@ -229,6 +229,18 @@ char		*	fname;
 	return(TRUE);
 }
 
+/*------------------------------------------------------------------------------
+ *
+ *----------------------------------------------------------------------------*/
+BOOL
+zip_get_aname(fname, sep, aname)
+const char	*	fname;
+char			sep;
+char		*	aname;
+{
+	return(zip_getname(fname, sep, aname, NULL));
+}
+
 /*------------------------------------------------------------------------------
  *	open() emulation
  *----------------------------------------------------------------------------*/
