@@ -39,7 +39,7 @@
 static int  write_buf __ARGS((int, char_u *, int));
 static void do_mlines __ARGS((void));
 #ifdef USE_OPT
-static char_u *opt_delet	__ARGS((char_u *, int, int, int, int));
+static char_u *opt_delet	__ARGS((char_u *, int, int, int, int, int, int, int));
 #endif
 #if defined(NT) && defined(USE_EXFILE) && defined(USE_MATOME)
 static int			do_encode	= FALSE;
@@ -248,13 +248,23 @@ readfile(fname, sfname, from, newfile, skip_lnum, nlines)
 #ifdef UNIX
 				if (perm < 0)
 #endif
+#if defined(NT) && defined(USE_EXFILE) && defined(USE_SHARE_CHECK)
+					if (ef_share_process(fname))
+					{
+						curbuf->b_p_ro = TRUE;
+						filemess(fname, (char_u *)"[Permission Denied]");
+					}
+					else
+						filemess(fname, (char_u *)"[New File]");
+#else
 					filemess(fname, (char_u *)"[New File]");
+#endif
 #ifdef UNIX
 				else
 					filemess(fname, (char_u *)"[Permission Denied]");
 #endif
 
-#ifdef KANJI
+#if 0
 			*code = JP_FILE;
 #endif
 		return FAIL;
@@ -319,7 +329,7 @@ readfile(fname, sfname, from, newfile, skip_lnum, nlines)
 			}
 #ifdef KANJI
 			if (firstpart && isupper(*code))
-				*code = judge_jcode(code, ptr, size);
+				*code = judge_jcode(code, &curbuf->b_p_ubig, ptr, size);
 # ifdef UCODE
 #  ifdef USE_OPT
 			if (!(p_opt & OPT_ORG_BINMODE) && curbuf->b_p_bin)
@@ -327,7 +337,15 @@ readfile(fname, sfname, from, newfile, skip_lnum, nlines)
 			else
 #  endif
 			if (toupper(*code) == JP_WIDE)	/* UNICODE */
-				size = wide2multi(ptr, size, firstpart);
+				size = wide2multi(ptr, size, curbuf->b_p_ubig, firstpart);
+			/* MS UTF8 */
+			else if (firstpart && toupper(*code) == JP_UTF8
+					&& (ptr[0] == 0xef && ptr[1] == 0xbb && ptr[2] == 0xbf))
+			{
+				filesize += 3;			/* count the number of characters */
+				size -= 3;
+				memmove((char *)ptr, (char *)ptr + 3, size);
+			}
 # endif
 #endif
 			filesize += size;				/* count the number of characters */
@@ -369,7 +387,7 @@ readfile(fname, sfname, from, newfile, skip_lnum, nlines)
 				{
 					char_u		tmp[81];
 
-					binaryconvsfrom(lnum, toupper(*code), &tail, line_start, ptr - line_start + 1, tmp);
+					binaryconvsfrom(lnum, (char_u)toupper(*code), &tail, line_start, ptr - line_start + 1, tmp);
 					tmp[80] = NUL;	/* end of line */
 					if (ml_append(lnum, tmp, strlen(tmp) + 1, newfile) == FAIL)
 					{
@@ -423,15 +441,17 @@ readfile(fname, sfname, from, newfile, skip_lnum, nlines)
 							break;
 						}
 						n = kanjiconvsfrom(line_start, len - 1,
-									tmp, n, NULL, toupper(*code), NULL);
+									tmp, n, NULL, (char_u)toupper(*code), NULL);
 						tmp[n] = NUL;	/* end of line */
 # ifdef USE_OPT
 						if (curbuf->b_p_opt & FOPT_EXTEND)
 						{
-							tmp = opt_delet(tmp,
+							tmp = opt_delet(tmp, 'R',
 										curbuf->b_p_opt & FOPT_EXPAND_TAB,
+										curbuf->b_p_opt & FOPT_ENCODE_TAB,
 										curbuf->b_p_opt & FOPT_DEL_SPC,
 										curbuf->b_p_opt & FOPT_REP_SPC,
+										curbuf->b_p_opt & FOPT_GAIJI,
 										(int)curbuf->b_p_ts);
 							n = strlen(tmp);
 						}
@@ -500,7 +520,7 @@ readfile(fname, sfname, from, newfile, skip_lnum, nlines)
 			{
 				char_u		tmp[81];
 
-				binaryconvsfrom(lnum, toupper(*code), &tail, line_start, ptr - line_start, tmp);
+				binaryconvsfrom(lnum, (char_u)toupper(*code), &tail, line_start, ptr - line_start, tmp);
 				tmp[80] = NUL;	/* end of line */
 				if (ml_append(lnum, tmp, strlen(tmp) + 1, newfile) == FAIL)
 					error = TRUE;
@@ -517,15 +537,17 @@ readfile(fname, sfname, from, newfile, skip_lnum, nlines)
 				break;
 			}
 			n = kanjiconvsfrom(line_start, ptr - line_start,
-										tmp, n, NULL, toupper(*code), NULL);
+									tmp, n, NULL, (char_u)toupper(*code), NULL);
 			tmp[n] = NUL;	/* end of line */
 # ifdef USE_OPT
 			if (curbuf->b_p_opt & FOPT_EXTEND)
 			{
-				tmp = opt_delet(tmp,
+				tmp = opt_delet(tmp, 'R',
 							curbuf->b_p_opt & FOPT_EXPAND_TAB,
+							curbuf->b_p_opt & FOPT_ENCODE_TAB,
 							curbuf->b_p_opt & FOPT_DEL_SPC,
 							curbuf->b_p_opt & FOPT_REP_SPC,
+							curbuf->b_p_opt & FOPT_GAIJI,
 							(int)curbuf->b_p_ts);
 				n = strlen(tmp);
 			}
@@ -594,7 +616,11 @@ readfile(fname, sfname, from, newfile, skip_lnum, nlines)
 #else
 					"\" %s%s%s%s%s%ld line%s, %ld character%s",
 #endif
+#if defined(NT) && defined(USE_EXFILE) && defined(USE_SHARE_CHECK)
+			curbuf->b_p_ro ? (ef_share_process(fname) ? "[READONLY]" : "[readonly] ") : "",
+#else
 			curbuf->b_p_ro ? "[readonly] " : "",
+#endif
 			incomplete ? "[Incomplete last line] " : "",
 			split ? "[long lines split] " : "",
 			error ? "[READ ERRORS] " : "",
@@ -779,10 +805,10 @@ buf_write(buf, fname, sfname, start, end, append, forceit, reset_changed)
 		new.st_dev = new.st_ino = 0;
 
 		/*
-		 * Unix semantics has it, that we may have a writable file, 
+		 * Unix semantics has it, that we may have a writable file,
 		 * that cannot be recreated with a simple open(..., O_CREAT, ) e.g:
-		 *  - the directory is not writable, 
-		 *  - the file may be a symbolic link, 
+		 *  - the directory is not writable,
+		 *  - the file may be a symbolic link,
 		 *  - the file may belong to another user/group, etc.
 		 *
 		 * For these reasons, the existing writable file must be truncated and
@@ -794,7 +820,7 @@ buf_write(buf, fname, sfname, start, end, append, forceit, reset_changed)
 			{
 				some_error = TRUE;			/* out of memory */
 				goto nobackup;
-			}			
+			}
 #ifdef KANJI
 			if (!stat(fileconvsto(backup), &new) &&
 						new.st_dev == old.st_dev && new.st_ino == old.st_ino)
@@ -830,9 +856,9 @@ buf_write(buf, fname, sfname, start, end, append, forceit, reset_changed)
 		if (backup == NULL || (bfd = open((char *)backup, O_WRONLY | O_CREAT, 0666)) < 0)
 #endif
 		{
-			/* 
+			/*
 			 * 'backupdir' starts with '>' or  no write/create permission
-			 * in current dirr: try again in p_bdir directory. 
+			 * in current dirr: try again in p_bdir directory.
 			 */
 			free(backup);
 			wp = gettail(fname);
@@ -902,7 +928,7 @@ nobackup:
 		/* if forceit and the file was read-only: make it writable */
 	if (forceit && (old.st_uid == getuid()) && perm >= 0 && !(perm & 0200))
  	{
-		perm |= 0200;	
+		perm |= 0200;
 #ifdef KANJI
 		(void)setperm(fileconvsto(fname), perm);
 #else
@@ -918,7 +944,7 @@ nobackup:
 /*
  * If we are not appending, the file exists, and the 'writebackup' or
  * 'backup' option is set, make a backup.
- * Do not make any backup, if "writebackup" and "backup" are 
+ * Do not make any backup, if "writebackup" and "backup" are
  * both switched off. This helps when editing large files on
  * almost-full disks. (jw)
  */
@@ -1053,7 +1079,7 @@ nexttry:
 	if (reset_changed && !newfile && !otherfile(ffname) && !(exiting && backup != NULL))
 		ml_preserve(buf, FALSE);
 
-	/* 
+	/*
 	 * We may try to open the file twice: If we can't write to the
 	 * file and forceit is TRUE we delete the existing file and try to create
 	 * a new one. If this still fails we may have lost the original file!
@@ -1083,7 +1109,7 @@ nexttry:
 #ifdef UNIX
 				/* we write to the file, thus it should be marked
 													writable after all */
-				perm |= 0200;		
+				perm |= 0200;
 				made_writable = TRUE;
 				if (old.st_uid != getuid() || old.st_gid != getgid())
 					perm &= 0777;
@@ -1151,8 +1177,18 @@ nexttry:
 #if defined(KANJI) && defined(UCODE)
 	if (toupper(*buf->b_p_jc) == JP_WIDE && !append && !curbuf->b_p_bin)	/* UNICODE */
 	{
-		short_u		w = 0xfeff;		/* BOM */
-		if (write_buf(fd, (char *)&w, (size_t)2) == FAIL)
+		char_u		w[2];
+		if (curbuf->b_p_ubig)
+		{
+			w[0] = 0xfe;
+			w[1] = 0xff;
+		}
+		else
+		{
+			w[0] = 0xff;
+			w[1] = 0xfe;
+		}
+		if (write_buf(fd, (char *)w, (size_t)2) == FAIL)
 			end = 0;
 		nchars = 2;
 	}
@@ -1176,7 +1212,7 @@ nexttry:
 		{
 			int			wlen;
 
-			tmpptr = binaryconvsto(toupper(*buf->b_p_jc), ptr + 1, &wlen);
+			tmpptr = binaryconvsto((char_u)toupper(*buf->b_p_jc), ptr + 1, &wlen, curbuf->b_p_ubig);
 			if (wlen && tmpptr)
 			{
 				if (write_buf(fd, tmpptr, wlen) == FAIL)
@@ -1196,17 +1232,19 @@ nexttry:
 
 			org = alloc(strlen(ptr + 1) + 1);
 			strcpy(org, ptr + 1);
-			org = opt_delet(org,
+			org = opt_delet(org, 'W',
 						buf->b_p_opt & FOPT_EXPAND_TAB,
+						curbuf->b_p_opt & FOPT_ENCODE_TAB,
 						buf->b_p_opt & FOPT_DEL_SPC,
 						buf->b_p_opt & FOPT_REP_SPC,
+						buf->b_p_opt & FOPT_GAIJI,
 						(int)buf->b_p_ts);
-			tmpptr = kanjiconvsto(org, toupper(*buf->b_p_jc));
+			tmpptr = kanjiconvsto(org, (char_u)toupper(*buf->b_p_jc), buf->b_p_ubig);
 			free(org);
 		}
 		else
 # endif
-		tmpptr = kanjiconvsto(ptr + 1, toupper(*buf->b_p_jc));
+		tmpptr = kanjiconvsto(ptr + 1, (char_u)toupper(*buf->b_p_jc), buf->b_p_ubig);
 		ptr = tmpptr - 1;
 #endif
 #if defined(KANJI) && defined(UCODE)
@@ -1222,7 +1260,7 @@ nexttry:
 				c = *++ptr;
 				wrk[0] = c;
 				wrk[1] = ptr[1];
-				if (wide2multi(wrk, 2, FALSE) == 1)
+				if (wide2multi(wrk, 2, curbuf->b_p_ubig, FALSE) == 1)
 				{
 					if (wrk[0] == NL)
 					{
@@ -1280,7 +1318,7 @@ nexttry:
 			if (toupper(*buf->b_p_jc) == JP_WIDE)	/* UNICODE */
 			{
 				wrk[0] = CR;
-				multi2wide(&wrk[0], &wrk[1], 1);
+				multi2wide(&wrk[0], &wrk[1], 1, buf->b_p_ubig);
 				s[0] = wrk[0];
 				s[1] = wrk[1];
 				++s;
@@ -1317,7 +1355,7 @@ nexttry:
 		if (toupper(*buf->b_p_jc) == JP_WIDE)	/* UNICODE */
 		{
 			wrk[0] = NL;
-			multi2wide(&wrk[0], &wrk[1], 1);
+			multi2wide(&wrk[0], &wrk[1], 1, buf->b_p_ubig);
 			s[0] = wrk[0];
 			s[1] = wrk[1];
 			++s;
@@ -1377,6 +1415,9 @@ nexttry:
 #ifdef MSDOS		/* the screen may be messed up by the "insert disk
 							in drive b: and hit return" message */
 	if (!exiting)
+# ifdef NT
+		if (!GuiWin)
+# endif
 		screenclear();
 #endif
 
@@ -1496,7 +1537,7 @@ nexttry:
 	if (!p_bk && backup != NULL && remove((char *)backup) != 0)
 #endif
 		EMSG("Can't delete backup file");
-	
+
 	goto nofail;
 
 fail:
@@ -1830,11 +1871,14 @@ vim_fgets(buf, size, fp, lnum)
 
 #ifdef USE_OPT
 	static char_u *
-opt_delet(buf, expand, delete, replace, ts)
+opt_delet(buf, readwrite, expand, entab, delete, replace, gaiji, ts)
 	char_u *buf;
+	int		readwrite;
 	int		expand;
+	int		entab;
 	int		delete;
 	int		replace;
+	int		gaiji;
 	int		ts;
 {
 	char_u		*	cp	= buf;
@@ -1842,21 +1886,90 @@ opt_delet(buf, expand, delete, replace, ts)
 	colnr_t			n;
 	colnr_t			col;
 	colnr_t			space = (-1);
+	colnr_t			vcol;
+	int				top = FALSE;
 
 	if (expand && ts < 1)
+	{
 		expand = FALSE;
+		entab  = FALSE;
+	}
 # ifdef KANJI
-	if (replace)
+	if (replace || gaiji)
 	{
 		col = 0;
 		while (buf[col])
 		{
-			if (ISkanji(buf[col]))
+			if (replace && ISkanji(buf[col]) && buf[col + 1]
+										&& isjpspace(buf[col], buf[col + 1]))
 			{
-				if (buf[col] == 0x81 && buf[col+1] == 0x40)
-					buf[col] = buf[col+1] = ' ';
+				buf[col] = buf[col+1] = ' ';
 				col += 2;
 			}
+			else if (gaiji && readwrite == 'R' && buf[col] == '#')
+			{
+				if (strlen(&buf[col]) >= 6 && buf[col + 5] == '#')
+				{
+					int			 cc = 0;
+					int			 i;
+
+					for (i = 0; i < 4; ++i)
+					{
+						if (!isxdigit(buf[col + 1 + i]))
+						{
+							cc = -1;
+							break;
+						}
+						if ('0' <= buf[col + 1 + i] && buf[col + 1 + i] <= '9')
+							cc = cc * 16 + buf[col + 1 + i] - '0';
+						else
+							cc = cc * 16 + toupper(buf[col + 1 + i]) - 'A' + 10;
+					}
+					if (cc == -1)
+						col++;
+					else
+					{
+						buf[col + 0]  = (cc & 0xff00) >> 8;
+						buf[col + 1]  =  cc & 0x00ff;
+						memmove(&buf[col + 2], &buf[col + 6],
+												strlen(&buf[col + 6]) + 1);
+						col += 2;
+					}
+				}
+				else
+					col++;
+			}
+			else if (gaiji && readwrite == 'W' && buf[col + 1]
+									&& 0xf0 <= buf[col] && buf[col] <= 0xf9)
+			{
+				char_u		*	p;
+				int			 	i;
+				int				c;
+
+				if ((p = alloc(strlen(buf) + 4 + 1)) == NULL)
+					gaiji = NUL;
+				else
+				{
+					memcpy(p, buf, col);
+					p[col + 0] = '#';
+					for (i = 1; i < 5; i++)
+					{
+						if (i & 1)
+							c = (buf[col + (i - 1) / 2] & 0xf0) >> 4;
+						else
+							c = (buf[col + (i - 1) / 2] & 0x0f);
+						p[col + i] = c > 9 ? 'A' + c - 10 : '0' + c;
+					}
+					p[col + 5] = '#';
+					memcpy(&p[col + 6], &buf[col + 2],
+											strlen(&buf[col + 2]) + 1);
+					col += 6;
+					free(buf);
+					cp = buf = p;
+				}
+			}
+			else if (ISkanji(buf[col]) && buf[col + 1])
+				col += 2;
 			else
 				col++;
 		}
@@ -1868,7 +1981,7 @@ opt_delet(buf, expand, delete, replace, ts)
 		while (buf[col])
 		{
 # ifdef KANJI
-			if (ISkanji(buf[col]))
+			if (ISkanji(buf[col]) && buf[col + 1])
 				col += 2;
 			else
 # endif
@@ -1884,18 +1997,53 @@ opt_delet(buf, expand, delete, replace, ts)
 			expand = FALSE;
 		}
 	}
-	n = col = 0;
+	vcol = n = col = 0;
 	while (buf[col])
 	{
-# ifdef KANJI
-		if (ISkanji(buf[col]))
+		if (entab && !top)
 		{
-			if (buf[col] == 0x81 && buf[col+1] == 0x40)
+			int			n_tabs;
+			int			n_spaces;
+			int			i;
+
+			if (buf[col] == ' ')
+			{
+				col++;
+				vcol += 1;
+				continue;
+			}
+			if (buf[col] == '\t')
+			{
+				col++;
+				vcol += ts - (vcol % ts);
+				continue;
+			}
+# ifdef KANJI
+			if (ISkanji(buf[col]) && buf[col + 1] && isjpspace(buf[col], buf[col + 1]))
+			{
+				vcol += 2;
+				col += 2;
+				continue;
+			}
+# endif
+			top = TRUE;
+			n_tabs = vcol / ts;
+			for (i = 0; i < n_tabs; i++)
+				cp[n++] = '\t';
+			n_spaces = vcol % ts;
+			for (i = 0; i < n_spaces; i++)
+				cp[n++] = ' ';
+		}
+# ifdef KANJI
+		if (ISkanji(buf[col]) && buf[col + 1])
+		{
+			if (isjpspace(buf[col], buf[col + 1]))
 				space = (space != (-1)) ? space : n;
 			else
 				space = (-1);
 			cp[n++] = buf[col++];
 			cp[n++] = buf[col++];
+			vcol += 2;
 		}
 		else
 # endif
@@ -1905,9 +2053,10 @@ opt_delet(buf, expand, delete, replace, ts)
 			int			i;
 
 			space = (space != (-1)) ? space : n;
+			n_spaces = ts - (vcol % ts);
+			vcol += n_spaces;
 			if (expand)
 			{
-				n_spaces = ts - (n % ts);
 				for (i = 0; i < n_spaces; i++)
 					cp[n++] = ' ';
 				col++;
@@ -1919,11 +2068,13 @@ opt_delet(buf, expand, delete, replace, ts)
 		{
 			space = (space != (-1)) ? space : n;
 			cp[n++] = buf[col++];
+			vcol += 1;
 		}
 		else
 		{
 			space = (-1);
 			cp[n++] = buf[col++];
+			vcol += 1;
 		}
 	}
 	if (delete && space != (-1) && space < n)
