@@ -119,6 +119,11 @@ static void		init_typestr __ARGS((void));
 static void		gotchars __ARGS((char_u *, int));
 static int		vgetorpeek __ARGS((int));
 static void		showmap __ARGS((struct mapblock *));
+#ifdef KANJI
+static int		show_special __ARGS((char_u *, int));
+static char_u * put_special __ARGS((char_u *));
+static int		convert_special __ARGS((char_u *));
+#endif
 
 /*
  * free and clear a buffer
@@ -924,6 +929,9 @@ vgetorpeek(advance)
 							&& !((State & (INSERT + CMDLINE)) && p_paste)
 							&& !(State == HITRETURN && (typestr[0] == CR || typestr[0] == ' ')))
 					{
+#ifdef NT
+						if (!(GuiWin && NoMap))
+#endif
 						for (mp = maplist.m_next; mp; mp = mp->m_next)
 						{
 							if ((mp->m_mode & ABBREV) || !(mp->m_mode & State))
@@ -1203,11 +1211,8 @@ domap(maptype, keys, mode)
 	while (*p)
 	{
 #ifdef KANJI
-		if (*p == Ctrl('V') && p[1] == 'F' && p[2] == 'n')
-		{
-			STRCPY(p, p + 2);
-			*p = 0xff;
-		}
+		if (convert_special(p))
+			p++;
 		else
 #endif
 		if (*p == Ctrl('V') && p[1] != NUL)
@@ -1220,21 +1225,17 @@ domap(maptype, keys, mode)
  */
 	if (haskey)
 	{
+#ifdef KANJI
+		convert_special(keys);
+#else
 		if (*keys == '#' && isdigit(*(keys + 1)))	/* function key */
 		{
-#ifdef KANJI
-			*keys= 0xff;
-			if (keys[1] == '0')
-				keys[1] = K_F10 & 0xff;
-			else
-				keys[1] += (K_F1 - '1') & 0xff;
-#else
 			if (*++keys == '0')
 				*keys = K_F10;
 			else
 				*keys += K_F1 - '1';
-#endif
 		}
+#endif
 		len = STRLEN(keys);
 		if (len > MAXMAPLEN)			/* maximum lenght of MAXMAPLEN chars */
 			return 1;
@@ -1414,7 +1415,11 @@ showmap(mp)
 		msg_outstr((char_u *)"i ");
 	else if (mp->m_mode & CMDLINE)
 		msg_outstr((char_u *)"c ");
+#ifdef KANJI
+	len = show_special(mp->m_keys, -1);	/* get length of what we have written */
+#else
 	len = msg_outtrans(mp->m_keys, -1);	/* get length of what we have written */
+#endif
 	do
 	{
 		msg_outchar(' ');				/* padd with blanks */
@@ -1609,6 +1614,24 @@ putescstr(fd, str, set)
 		 * A space has to be escaped with a backslash to
 		 * prevent it to be misinterpreted in doset().
 		 */
+#ifdef KANJI
+		if (!set && *str == K_SPECIAL)
+		{
+			char_u	*	p = put_special(str);
+
+			if (p != NULL)
+			{
+				while (*p)
+				{
+					if (putc(*p, fd) < 0)
+						return FAIL;
+					p++;
+				}
+				str++;
+				continue;
+			}
+		}
+#endif
 		if (*str < ' ' || *str > '~' || (*str == ' ' && !set))
 		{
 			if (putc(Ctrl('V'), fd) < 0)
@@ -1624,3 +1647,126 @@ putescstr(fd, str, set)
 	}
 	return OK;
 }
+
+#ifdef KANJI
+static struct {
+	char			*	name;
+	int_u				key;
+} special_keys[] = {
+	{	"#[UP]",		K_UARROW	},
+	{	"#[U]",			K_UARROW	},
+	{	"#[DOWN]",		K_DARROW	},
+	{	"#[D]",			K_DARROW	},
+	{	"#[LEFT]",		K_LARROW	},
+	{	"#[L]",			K_LARROW	},
+	{	"#[RIGHT]",		K_RARROW	},
+	{	"#[R]",			K_RARROW	},
+	{	"#[SUP]",		K_SUARROW	},
+	{	"#[SDOWN]",		K_SDARROW	},
+	{	"#[SLEFT]",		K_SLARROW	},
+	{	"#[SRIGHT]",	K_SRARROW	},
+	{	"#[F01]",		K_F1		},
+	{	"#[F02]",		K_F2		},
+	{	"#[F03]",		K_F3		},
+	{	"#[F04]",		K_F4		},
+	{	"#[F05]",		K_F5		},
+	{	"#[F06]",		K_F6		},
+	{	"#[F07]",		K_F7		},
+	{	"#[F08]",		K_F8		},
+	{	"#[F09]",		K_F9		},
+	{	"#[F10]",		K_F10		},
+	{	"#[F11]",		K_SF1		},
+	{	"#[F12]",		K_SF2		},
+	{	"#[F13]",		K_SF3		},
+	{	"#[F14]",		K_SF4		},
+	{	"#[F15]",		K_SF5		},
+	{	"#[F16]",		K_SF6		},
+	{	"#[F17]",		K_SF7		},
+	{	"#[F18]",		K_SF8		},
+	{	"#[F19]",		K_SF9		},
+	{	"#[F20]",		K_SF10		},
+	{	"#[HELP]",		K_HELP		},
+	{	"#[UNDO]",		K_UNDO		},
+	{	"#1",			K_F1		},
+	{	"#2",			K_F2		},
+	{	"#3",			K_F3		},
+	{	"#4",			K_F4		},
+	{	"#5",			K_F5		},
+	{	"#6",			K_F6		},
+	{	"#7",			K_F7		},
+	{	"#8",			K_F8		},
+	{	"#9",			K_F9		},
+	{	"#0",			K_F10		},
+	{	NULL,			0			},
+};
+
+static int
+show_special(keys, len)
+	char_u	*keys;
+	int		 len;
+{
+	int				i;
+	char_u			buf[CMDBUFFSIZE + 1];
+	char_u		*	p;
+	char_u		*	w;
+
+	p = buf;
+	while (*keys)
+	{
+		if (*keys == K_SPECIAL)
+		{
+			for (w = NULL, i = 0; special_keys[i].key != 0; i++)
+			{
+				if ((special_keys[i].key & 0xff) == keys[1])
+				{
+					w = special_keys[i].name;
+					while (*w)
+						*p++ = *w++;
+					break;
+				}
+			}
+			if (w != NULL)
+			{
+				keys += 2;
+				continue;
+			}
+		}
+		*p++ = *keys++;
+	}
+	*p = NUL;
+	return(msg_outtrans(buf, len));
+}
+
+static char_u *
+put_special(keys)
+	char_u	*keys;
+{
+	int				i;
+
+	for (i = 0; special_keys[i].key != 0; i++)
+	{
+		if ((special_keys[i].key & 0xff) == keys[1])
+			return(special_keys[i].name);
+	}
+	return(NULL);
+}
+
+static int
+convert_special(keys)
+	char_u	*keys;
+{
+	int				i;
+
+	for (i = 0; special_keys[i].name != NULL; i++)
+	{
+		if (strncmp(special_keys[i].name, keys, strlen(special_keys[i].name)) == 0)
+		{
+			*keys = K_SPECIAL;
+			keys[1] = special_keys[i].key & 0xff;
+			STRCPY(&keys[2], &keys[strlen(special_keys[i].name)]);
+			return(1);
+		}
+	}
+	return(0);
+}
+#endif
