@@ -66,6 +66,7 @@ static unsigned char	*	IObuffer;
 static long					IOsize;
 static unsigned char	*	IOstart;
 static long					IOrest;
+static int					o_ubig;
 
 /*==============================================================================
  *	file parameter
@@ -248,10 +249,10 @@ char		*	fname;
 	if ((IOrest = read(fd, (char *)IObuffer, (size_t)IOsize)) <= 0)
 		return(fd);
 	if (isupper(JP_KEY))
-		JP_KEY = judge_jcode(&JP_KEY, IObuffer, IOrest);
+		JP_KEY = judge_jcode(&JP_KEY, &o_ubig, IObuffer, IOrest);
 #ifdef NT
 	if (toupper(JP_KEY) == JP_WIDE)	/* UNICODE */
-		IOrest = wide2multi(IObuffer, IOrest, TRUE);
+		IOrest = wide2multi(IObuffer, IOrest, o_ubig, TRUE);
 #endif
 	return(fd);
 }
@@ -308,7 +309,7 @@ retry:
 			lp->lp[lp->size] = '\0';
 		}
 		n = kanjiconvsfrom(IOstart, ptr - IOstart,
-						&lp->lp[lp->size], n, NULL, toupper(JP_KEY), NULL);
+					&lp->lp[lp->size], n, NULL, (char_u)toupper(JP_KEY), NULL);
 		lp->size += n;
 		lp->lp[lp->size] = '\0';	/* end of line */
 		lp->rest = lp->rest - n;
@@ -335,7 +336,7 @@ retry:
 			IOrest += rest;
 #ifdef NT
 			if (toupper(JP_KEY) == JP_WIDE)	/* UNICODE */
-				IOrest = wide2multi(IObuffer, IOrest, FALSE);
+				IOrest = wide2multi(IObuffer, IOrest, o_ubig, FALSE);
 #endif
 			if (IOrest == 0)		/* end of file */
 				return(0);
@@ -662,8 +663,8 @@ char				*	fname;
 								printf("%s:", fname);
 						}
 						if (o_num)
-							printf("%d:", lp->num);
-						p = kanjiconvsto(lp->lp, JP_DISP);
+							printf("%ld:", lp->num);
+						p = kanjiconvsto(lp->lp, JP_DISP, TRUE);
 						printf("%s\n", p);
 						free(p);
 						pnum = lp->num;
@@ -691,7 +692,7 @@ char				*	fname;
 			else
 				printf("%s:", fname);
 		}
-		printf("%d\n", counter);
+		printf("%ld\n", counter);
 	}
 	close(o_fd);
 	return(found);
@@ -718,6 +719,12 @@ char			**	argv;
 	 */
 	while (argc >= 1 && argv[0][0] == '-')
 	{
+		if (argv[0][1] == '-' && argv[0][2] == '\0')
+		{
+			++ argv;
+			-- argc;
+			break;
+		}
 		skip = FALSE;
 		for (cp = argv[0] + 1; *cp && skip == FALSE; cp++)
 		{
@@ -910,6 +917,7 @@ char			**	argv;
 			fprintf(stderr, "memory allocation error.\n");
 		exit(2);
 	}
+	memset(line, 0, sizeof(LINE *) * ((o_count > o_after ? o_count : o_after) + o_before));
 	if ((linep = (LINE *)malloc(sizeof(LINE) * ((o_count > o_after ? o_count : o_after) + o_before))) == NULL)
 	{
 		if (!o_noerr)
@@ -1023,7 +1031,7 @@ char			**	argv;
 # endif
 				for (j = 0; result[j] != NULL; j++)
 				{
-					rtn != do_winsearch(result[j]);
+					rtn |= do_winsearch(result[j]);
 					free(result[j]);
 				}
 				free(result);
@@ -1061,7 +1069,7 @@ char			**	argv;
 # endif
 					for (j = 0; result[j] != NULL; j++)
 					{
-						rtn != do_winsearch(result[j]);
+						rtn |= do_winsearch(result[j]);
 						free(result[j]);
 					}
 					free(result);
@@ -1072,7 +1080,7 @@ char			**	argv;
 				}
 			}
 			else
-				rtn != do_winsearch(argv[i]);
+				rtn |= do_winsearch(argv[i]);
 #else
 			rtn |= do_grep(argv[i]);
 #endif
@@ -1133,7 +1141,11 @@ char			**	argv;
 	argv =  &argv[rtn];
 
 #ifdef XARGS
+# ifdef NT
 	if (dir_cnt == 0 && argc != 0)
+# else
+	if (argc != 0)
+# endif
 	{
 		xargs(&argc, &argv);
 		argc -= rtn;
@@ -1277,7 +1289,7 @@ char			*	ext;
 		for (j = 0; result[j] != NULL; j++)
 		{
 			multi = TRUE;
-			rtn != do_winsearch(result[j]);
+			rtn |= do_winsearch(result[j]);
 			free(result[j]);
 		}
 		free(result);
