@@ -37,6 +37,63 @@ usage(n)
 								(char_u *)"Argument missing\n",		/* 2 */
 								};
 
+#ifdef NT
+	if (!SubSysCon)
+	{
+		static char_u			msg[8192];
+		int						pos = 0;
+
+		pos += wsprintf(&msg[pos],
+# ifdef KANJI
+				"    %s\r\nPorted to W32-GUI and Japanized Ver. %s\n\n",
+					longVersion,
+					longJpVersion
+# else
+					"%s",
+					longVersion
+# endif
+				);
+		if (n <= 2)
+			pos += wsprintf(&msg[pos], (char *)errors[n]);
+		pos += wsprintf(&msg[pos], "usage:");
+		for (i = 0; ; ++i)
+		{
+			pos += wsprintf(&msg[pos], " vim [options] ");
+			pos += wsprintf(&msg[pos], (char *)use[i]);
+			if (i == (sizeof(use) / sizeof(char_u *)) - 1)
+				break;
+			pos += wsprintf(&msg[pos], "     or:");
+		}
+		pos += wsprintf(&msg[pos], "\noptions:\t-v\t\treadonly mode (view)\n");
+		pos += wsprintf(&msg[pos], "\t-n\t\tno swap file, use memory only\n");
+		pos += wsprintf(&msg[pos], "\t-b\t\tbinary mode\n");
+		pos += wsprintf(&msg[pos], "\t-r\t\trecovery mode\n");
+		pos += wsprintf(&msg[pos], "\t-T terminal\tset terminal type\n");
+		pos += wsprintf(&msg[pos], "\t-o[N]\t\topen N windows (def: one for each file)\n");
+		pos += wsprintf(&msg[pos], "\t+\t\tstart at end of file\n");
+		pos += wsprintf(&msg[pos], "\t+lnum\t\tstart at line lnum\n");
+		pos += wsprintf(&msg[pos], "\t-c command\texecute command first\n");
+		pos += wsprintf(&msg[pos], "\t-s scriptin\t\tread commands from script file\n");
+		pos += wsprintf(&msg[pos], "\t-w scriptout\twrite commands in script file\n");
+# if defined(USE_EXFILE) && defined(USE_MATOME)
+		pos += wsprintf(&msg[pos], "\t-X\t\tno use Extend File System\n");
+# endif
+# ifdef KANJI
+		pos += wsprintf(&msg[pos], "\t-k FileCode\tfile jcode set\n");
+		pos += wsprintf(&msg[pos], "\t-K KanjiCode\tjmask code set\n");
+# endif
+		pos += wsprintf(&msg[pos], "\t-B script\t\tbenchmark commands in script file\n");
+		pos += wsprintf(&msg[pos], "\t-I section\t\tsection name in .ini file\n");
+		MessageBox(NULL, msg,
+# ifdef KANJI
+					JpVersion,
+# else
+					Version,
+# endif
+				MB_OK);
+		mch_windexit(1);
+	}
+#endif
 #ifndef notdef
 	if (n <= 2)
 #endif
@@ -65,6 +122,17 @@ usage(n)
 	fprintf(stderr, "\t\t-c command\texecute command first\n");
 	fprintf(stderr, "\t\t-s scriptin\tread commands from script file\n");
 	fprintf(stderr, "\t\t-w scriptout\twrite commands in script file\n");
+#if defined(NT) && defined(USE_EXFILE) && defined(USE_MATOME)
+	fprintf(stderr, "\t\t-X\t\tno use Extend File System\n");
+#endif
+#ifdef KANJI
+	fprintf(stderr, "\t\t-k FileCode\tfile jcode set\n");
+	fprintf(stderr, "\t\t-K KanjiCode\tjmask code set\n");
+#endif
+#ifdef NT
+	fprintf(stderr, "\t\t-B script\tbenchmark commands in script file\n");
+	fprintf(stderr, "\t\t-I section\tsection name in .ini file\n");
+#endif
 	mch_windexit(1);
 }
 
@@ -139,7 +207,7 @@ main(argc, argv)
 	 *		'-T terminal'	terminal name
 	 */
 	while (argc > 1 && ((c = argv[0][0]) == '+' || (c == '-' &&
-			strchr("vnbrxocswTdRKkhB", c = argv[0][1]) != NULL && c != NUL)))
+			strchr("vnbrxocswTdRKkhBIX", c = argv[0][1]) != NULL && c != NUL)))
 	{
 		--argc;
 		switch (c)
@@ -205,6 +273,11 @@ main(argc, argv)
 			usage(9);
 			break;
 #endif
+#if defined(NT) && defined(USE_EXFILE) && defined(USE_MATOME)
+		case 'X':
+			NoEFS = !NoEFS;
+			break;
+#endif
 
 		default:	/* options with argument */
 			++argv;
@@ -281,6 +354,9 @@ main(argc, argv)
 						usage(9);
 				}
 				break;
+			case 'I':
+				STRCPY(GuiIni, argv[0]);
+				break;
 #endif
 
 		/*	case 'd':		This is ignored as it is handled in check_win() */
@@ -582,7 +658,10 @@ main(argc, argv)
 
 #ifdef NT
 	if (GuiWin)
+	{
 		++no_wait_return;
+		InitCommand();
+	}
 #endif
 	if (recoverymode)					/* do recover */
 	{
