@@ -824,7 +824,7 @@ mf_read(mfp, hp)
 	page_size = mfp->mf_page_size;
 	offset = page_size * hp->bh_bnum;
 	size = page_size * hp->bh_page_count;
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__bsdi__)
 	if (lseek(mfp->mf_fd, (off_t)offset, SEEK_SET) != offset)
 #else
 	if (lseek(mfp->mf_fd, offset, SEEK_SET) != offset)
@@ -885,7 +885,7 @@ mf_write(mfp, hp)
 			hp2 = hp;
 
 		offset = page_size * nr;
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__bsdi__)
 		if (lseek(mfp->mf_fd, (off_t)offset, SEEK_SET) != offset)
 #else
 		if (lseek(mfp->mf_fd, offset, SEEK_SET) != offset)
