@@ -426,6 +426,10 @@ inschar(c)
 
 	old = ml_get(lnum);
 	oldlen = STRLEN(old) + 1;
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+	if (p_syn && GuiWin)
+		syn_inschar(old, col);
+#endif
 
 	rir0 = (State == REPLACE && p_ri && col == 0);
 	if (rir0 || State != REPLACE || *(old + col) == NUL)
@@ -447,7 +451,7 @@ inschar(c)
 	p = new + col;
 #ifdef KANJI
 	new[oldlen + extra - 1] = NUL;
-	if (p_jrep && State == REPLACE && !ISkanji(c) && ISkanji(gchar_cursor()))
+	if (!p_jrep && State == REPLACE && !ISkanji(c) && ISkanji(gchar_cursor()))
 	{
 		memmove((char *)p, (char *)old + col, (size_t)(oldlen - col));
 		if (rir0)					/* reverse replace in column 0 */
@@ -455,7 +459,7 @@ inschar(c)
 		else
 			*(p + 1) = ' ';
 	}
-	else if (p_jrep && State == REPLACE && ISkanji(c) && !ISkanji(gchar_cursor()))
+	else if (!p_jrep && State == REPLACE && ISkanji(c) && !ISkanji(gchar_cursor()))
 	{
 		if (gchar_cursor() == NUL)
 		{
@@ -536,6 +540,10 @@ inschar(c)
 		++curwin->w_cursor.col;
 	else if (State == REPLACE && !rir0)	/* reverse replace mode: cursor left */
 		--curwin->w_cursor.col;
+#endif
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+	if (p_syn && GuiWin)
+		syn_inschar(new, col);
 #endif
 	CHANGED;
 }
@@ -583,6 +591,10 @@ delchar(fixpos)
 
 	old = ml_get(lnum);
 	oldlen = STRLEN(old);
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+	if (p_syn && GuiWin)
+		syn_delchar(old, col);
+#endif
 
 	if (col >= oldlen)	/* can't do anything (happens with replace mode) */
 		return FAIL;
@@ -734,7 +746,7 @@ skipspace(pp)
     register char_u *p;
     
 #ifdef KANJI
-    for(p = *pp; 1; ++p)
+    for(p = *pp; *p; ++p)
 	{
 		if (*p == ' ' || *p == '\t')
 			continue;
@@ -1038,9 +1050,18 @@ home_replace(src, dst, dstlen)
 		STRNCPY(dst, src, (size_t)dstlen);
 	else
 	{
+#ifndef notdef
+		if (home[len - 1] == '\\' || home[len - 1] == '/')
+			len--;
+#endif
 		skipspace(&src);
 		while (*src && dstlen > 0)
 		{
+#ifndef notdef
+			if (strlen(src) > len && src[len] != '\\' && src[len] != '/')
+				;
+			else
+#endif
 			if (STRNCMP(src, home, len) == 0)
 			{
 				src += len;
