@@ -196,6 +196,26 @@ getcount:
 # endif
 		if (curbuf->b_p_fc && curbuf->fepmode && !fep_get_mode())
 			fep_force_on();
+		switch (*curbuf->b_p_ji) {
+		case 'j':
+			if (!p_ja)
+				break;
+			/* no break */
+		case 'J':
+			if (!fep_get_mode())
+				fep_force_on();
+			break;
+		case 'a':
+			if (!p_ja)
+				break;
+			/* no break */
+		case 'A':
+			if (fep_get_mode())
+				fep_force_off();
+			break;
+		default:
+			break;
+		}
 	}
 #endif
 	if (ctrl_w)
@@ -454,6 +474,9 @@ retry_input:
 
 	  case Ctrl('L'):
 		CHECKCLEAROP;
+#ifdef NT
+		wincmd_redraw();
+#endif
 		updateScreen(CLEAR);
 		break;
 
@@ -592,17 +615,30 @@ sow:				if (i == 0)
 				}
 			}
 			else
+			{
+# ifdef USE_TAGEX
+				if (p_tagex && i == 0 && c == Ctrl(']') && col > 0)
+				{
+					if (ptr[col - 1] == '~')
+						stuffcharReadbuff('~');
+				}
+# endif
 				while (!ISkanji(ptr[col])
 					&& (i == 0 ? isidchar(ptr[col]) :
 								(ptr[col] != NUL && !iswhite(ptr[col]))))
+				{
+					stuffcharReadbuff(ptr[col]);
+					++col;
+				}
+			}
 #else
 			while (i == 0 ? isidchar(ptr[col]) :
 								(ptr[col] != NUL && !iswhite(ptr[col])))
-#endif
 			{
 				stuffcharReadbuff(ptr[col]);
 				++col;
 			}
+#endif
 			if ((c == '*' || c == '#') && i == 0)
 				stuffReadbuff((char_u *)"\\>");
 			stuffReadbuff((char_u *)"\n");
@@ -1795,6 +1831,11 @@ gotofile:
 								NameBuff[line - name - 2] = NUL;
 								STRCPY(filebuf, &NameBuff[1]);
 							}
+							if (getperm(filebuf) == (-1) || atol(linebuf) == 0)
+							{
+								CLEAROPBEEP;
+								goto error;
+							}
 # if 1
 							{
 								static WIN		*	swin = NULL;
@@ -1881,6 +1922,215 @@ error:
 						}
 						break;
 #endif
+#ifndef notdef
+						/*
+						 * "gc": count charactor
+						 */
+			case 'c':	if (VIsual.lnum)
+						{
+							linenr_t		line;
+							long			count = 0;
+
+							curbuf->b_startop = VIsual;
+							if (lt(curbuf->b_startop, curwin->w_cursor))
+							{
+								curbuf->b_endop = curwin->w_cursor;
+								curwin->w_cursor = curbuf->b_startop;
+							}
+							else
+							{
+								curbuf->b_endop = curbuf->b_startop;
+								curbuf->b_startop = curwin->w_cursor;
+							}
+							if (ISkanjiFpos(&curbuf->b_endop))
+								curbuf->b_endop.col++;
+							if (curbuf->b_startop.lnum == curbuf->b_endop.lnum
+															&& mtype == MCHAR)
+								count = curbuf->b_endop.col - curbuf->b_startop.col + 1;
+							else
+							{
+								for (line = curbuf->b_startop.lnum;
+										line <= curbuf->b_endop.lnum; line++)
+								{
+									if (mtype == MLINE)
+										count += strlen(ml_get(line)) + (curbuf->b_p_tx ? 2 : 1);
+									else if (line == curbuf->b_startop.lnum)
+										count += strlen(ml_get(line)) - curbuf->b_startop.col + (curbuf->b_p_tx ? 2 : 1);
+									else if (line == curbuf->b_endop.lnum)
+										count += curbuf->b_endop.col + 1;
+									else
+										count += strlen(ml_get(line)) + (curbuf->b_p_tx ? 2 : 1);
+								}
+							}
+							VIsual.lnum = 0;
+							updateScreen(NOT_VALID);
+							setcursor();
+							updateScreen(CURSUPD);
+							smsg((char_u *)"%ld character%s.", count, plural(count));
+						}
+						break;
+
+						/*
+						 * "gj": logical line down
+						 */
+			case 'j':	if (curwin->w_p_wrap)
+						{
+							int				vpos	= (curwin->w_curswant + (curwin->w_p_nu ? 8 : 0)) % Columns;
+							int				vcol	= getvcol(curwin, &curwin->w_cursor, 99);
+							int				tcol;
+							FPOS			cursor;
+
+							if (vcol % Columns != vpos)
+								vcol = (vcol / Columns) * Columns + vpos;
+							while (Prenum1)
+							{
+								cursor.lnum = curwin->w_cursor.lnum;
+								cursor.col  = strlen(ml_get(curwin->w_cursor.lnum));
+								tcol = getvcol(curwin, &cursor, 99);
+								if (tcol <= (vcol + Columns))
+								{
+									Prenum1 --;
+									if (tcol <= ((vcol / Columns) + 1) * Columns)
+									{
+										if (curwin->w_cursor.lnum
+													< curwin->w_buffer->b_ml.ml_line_count)
+										{
+											curwin->w_cursor.lnum ++;
+											curwin->w_cursor.col = 0;
+											vcol = vpos;
+										}
+										else
+											break;
+									}
+									else if ((tcol % Columns) <= vpos)
+										vcol = tcol;
+								}
+								else
+								{
+									vcol += Columns;
+									Prenum1 --;
+								}
+							}
+							curwin->w_curswant = (vcol / Columns) * Columns + vpos - (curwin->w_p_nu ? 8 : 0);
+							curwin->w_cursor.col = vcol2col(curwin, curwin->w_cursor.lnum, vcol, NULL, curwin->w_p_nu, Columns);
+							curs_columns(TRUE);
+						}
+						else
+							onedown(Prenum1);
+						break;
+
+						/*
+						 * "gk": logical line up
+						 */
+			case 'k':	if (curwin->w_p_wrap)
+						{
+							int				vpos	= (curwin->w_curswant + (curwin->w_p_nu ? 8 : 0)) % Columns;
+							int				vcol	= getvcol(curwin, &curwin->w_cursor, 99);
+							FPOS			cursor;
+
+							while (Prenum1)
+							{
+								if (vcol >= Columns)
+									vcol = ((vcol / Columns) - 1) * Columns + vpos;
+								else if (curwin->w_cursor.lnum > 1)
+								{
+									curwin->w_cursor.lnum--;
+									curwin->w_cursor.col = strlen(ml_get(curwin->w_cursor.lnum));
+									cursor = curwin->w_cursor;
+									vcol = getvcol(curwin, &cursor, 99);
+									if (vcol > Columns)
+										vcol = (vcol / Columns) * Columns + vpos;
+									else
+										vcol = vpos;
+								}
+								else
+									break;
+								Prenum1--;
+							}
+							curwin->w_curswant = (vcol / Columns) * Columns + vpos - (curwin->w_p_nu ? 8 : 0);
+							curwin->w_cursor.col = vcol2col(curwin, curwin->w_cursor.lnum, vcol, NULL, curwin->w_p_nu, Columns);
+							curs_columns(TRUE);
+						}
+						else
+							oneup(Prenum1);
+						break;
+
+						/*
+						 * "g0": logical line top
+						 */
+			case '0':	if (curwin->w_p_wrap)
+						{
+							int				vcol	= getvcol(curwin, &curwin->w_cursor, 99);
+
+							vcol = (vcol / Columns) * Columns;
+							curwin->w_curswant = (vcol / Columns) * Columns - (curwin->w_p_nu ? 8 : 0);
+							curwin->w_cursor.col = vcol2col(curwin, curwin->w_cursor.lnum, vcol, NULL, curwin->w_p_nu, Columns);
+							curs_columns(TRUE);
+						}
+						else
+						{
+							mtype = MCHAR;
+							mincl = FALSE;
+							beginline(FALSE);
+						}
+						break;
+
+						/*
+						 * "g^": logical line top character
+						 */
+			case '^':	if (curwin->w_p_wrap)
+						{
+							int				vcol	= getvcol(curwin, &curwin->w_cursor, 99);
+							char_u		*	p;
+
+							vcol = (vcol / Columns) * Columns;
+							curwin->w_curswant = (vcol / Columns) * Columns - (curwin->w_p_nu ? 8 : 0);
+							curwin->w_cursor.col = vcol2col(curwin, curwin->w_cursor.lnum, vcol, NULL, curwin->w_p_nu, Columns);
+							p = ml_get(curwin->w_cursor.lnum);
+							while (p[curwin->w_cursor.col])
+							{
+								if (!isspace(p[curwin->w_cursor.col]))
+									break;
+								curwin->w_cursor.col++;
+							}
+							if (p[curwin->w_cursor.col] == NUL)
+							{
+								if (ISkanjiPosition(p, curwin->w_cursor.col) == 2)
+									curwin->w_cursor.col--;
+								curwin->w_cursor.col--;
+							}
+							curs_columns(TRUE);
+						}
+						else
+						{
+							mtype = MCHAR;
+							mincl = FALSE;
+							beginline(TRUE);
+						}
+						break;
+
+						/*
+						 * "g$": logical line bottom
+						 */
+			case '$':	if (curwin->w_p_wrap)
+						{
+							int				vcol	= getvcol(curwin, &curwin->w_cursor, 99);
+
+							vcol = ((vcol / Columns) + 1) * Columns - 1;
+							curwin->w_curswant = ((vcol / Columns) + 1) * Columns - (curwin->w_p_nu ? 8 : 0) - 1;
+							curwin->w_cursor.col = vcol2col(curwin, curwin->w_cursor.lnum, vcol, NULL, curwin->w_p_nu, Columns);
+							vcol = getvcol(curwin, &curwin->w_cursor, 99);
+							curs_columns(TRUE);
+						}
+						else
+						{
+							mtype = MCHAR;
+							mincl = TRUE;
+							curwin->w_curswant = MAXCOL;
+							onedown((long)(Prenum1 - 1));
+						}
+						break;
+#endif
 
 			default:	CLEAROPBEEP;
 						break;
