@@ -557,6 +557,9 @@ dodelete()
 			dellines(nlines, TRUE, TRUE);
 		}
 		u_clearline();	/* "U" command should not be possible after "dd" */
+#ifndef notdef		/* quick hack */
+		if (!(nlines > 1 && operator == CHANGE))
+#endif
 		beginline(TRUE);
 	}
 	else if (nlines == 1)		/* delete characters within one line */
@@ -1575,6 +1578,10 @@ dojoin(insert_space, redraw)
 #ifdef KANJI
 			if (ISkanji(*next) && ISkanjiPosition(curr, currsize) == 2)
 				spaces = 0;
+			else if (*next == NUL && ISkanjiPosition(curr, currsize) == 2)
+				spaces = 0;
+			else if (!p_jjs && (ISkanji(*next) || ISkanjiPosition(curr, currsize) == 2))
+				spaces = 0;
 			else
 #endif
 			if (p_js && strchr(".!?", endcurr1) != NULL)
