@@ -71,23 +71,26 @@ struct param
 
 #ifdef KANJI
 # define PV_JC		35
+# ifdef UCODE
+#  define PV_UB		37
+# endif
 #endif
 #ifdef CRMARK
-# define PV_CR		37
+# define PV_CR		39
 #endif
 #ifdef TRACK
-# define PV_TRS		39
-# define PV_TT		41
+# define PV_TRS		41
+# define PV_TT		43
 #endif
 #ifdef FEPCTRL
-#define PV_FC		43
-# define PV_JI		45
+#define PV_FC		45
+# define PV_JI		47
 #endif
 #ifdef USE_OPT
-# define PV_OPT		47
+# define PV_OPT		49
 #endif
 #if defined(NT) && defined(USE_EXFILE) && defined(USE_MATOME)
-# define PV_EC		49
+# define PV_EC		51
 #endif
 
 /*
@@ -120,6 +123,7 @@ static struct param params[] =
 		{"columns",		"co",	P_NUM,				(char_u *)&Columns},
 		{"compatible",	"cp",	P_BOOL,				(char_u *)&p_cp},
 #ifdef CRMARK
+		{"crchar",		"cc",	P_STRING,			(char_u *)&p_cc},
 		{"crmark",		"cm",	P_BOOL|P_IND,		(char_u *)PV_CR},
 #endif
 #if defined(NT) && defined(USE_MATOME)
@@ -165,6 +169,9 @@ static struct param params[] =
 		{"ignorecase",	"ic",	P_BOOL,				(char_u *)&p_ic},
 		{"insertmode",	"im",	P_BOOL,				(char_u *)&p_im},
 		{"joinspaces", 	"js",	P_BOOL,				(char_u *)&p_js},
+#if defined(KANJI) && defined(UCODE)
+		{"jbigendian",	"jb",	P_BOOL|P_IND,		(char_u *)PV_UB},
+#endif
 #ifdef KANJI
 		{"jcode",		"jc",  	P_STRING|P_IND,		(char_u *)PV_JC},
 #endif
@@ -178,6 +185,7 @@ static struct param params[] =
 		{"jinsertmode", "ji",	P_STRING|P_IND,		(char_u *)PV_JI},
 #endif
 #ifdef KANJI
+		{"jjoinspaces",	"jjs",	P_BOOL,				(char_u *)&p_jjs},
 		{"jkanaconv",	"jkc", 	P_BOOL,				(char_u *)&p_jkc},
 		{"jmask",		"jm",  	P_STRING,			(char_u *)&p_jp},
 		{"jreplace",	"jrep",	P_BOOL,				(char_u *)&p_jrep},
@@ -242,7 +250,15 @@ static struct param params[] =
 		{"sourceany",	NULL,	P_BOOL,				(char_u *)NULL},
 		{"splitbelow",	"sb",	P_BOOL,				(char_u *)&p_sb},
 		{"suffixes",	"su",	P_STRING,			(char_u *)&p_su},
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+		{"syntax",		"syn",	P_BOOL,				(char_u *)&p_syn},
+		{"syntype",		"syt",	P_STRING,			(char_u *)&p_synt},
+		{"synlines",	"syl",	P_NUM,				(char_u *)&p_synl},
+#endif
 		{"tabstop", 	"ts",	P_NUM|P_IND,		(char_u *)PV_TS},
+#ifdef USE_TAGEX
+		{"tagex",		"te",	P_BOOL,				(char_u *)&p_tagex},
+#endif
 		{"taglength",	"tl",	P_NUM,				(char_u *)&p_tl},
 		{"tagrelative",	"tr",	P_BOOL,				(char_u *)&p_tr},
 		{"tags",		NULL,	P_STRING|P_EXPAND,	(char_u *)&p_tags},
@@ -398,10 +414,14 @@ set_init()
 		if (p != NULL)
 		{
 			if ((STRCMP(p, "ja_JP.JIS") == 0)
+					|| (STRCMP(p, "ja_JP.ISO-2022-JP") == 0)
 					|| (STRCMP(p, "ja_JP.jis7") == 0))
 				STRCPY(p_jp_init, "JJJ");
-			else if ((STRCMP(p, "ja_JP.EUC") == 0)
+			else if ((STRCMP(p, "ja_JP.eucJP") == 0)
+					|| (STRCMP(p, "ja_JP.EUC") == 0)
+					|| (STRCMP(p, "ja") == 0)
 					|| (STRCMP(p, "japanese") == 0)
+					|| (STRCMP(p, "japanese.euc") == 0)
 					|| (STRCMP(p, "ja_JP.ujis") == 0))
 				STRCPY(p_jp_init, "EEE");
 			else if ((STRCMP(p, "ja_JP.SJIS") == 0)
@@ -464,6 +484,17 @@ set_init()
 		buf[0] = JP_FILE;
 		buf[1] = '\0';
 		curbuf->b_p_jc = strsave(buf);
+# ifdef UCODE
+		{
+			short_u		w = 0xfeff;		/* BOM */
+
+			memmove((char *)buf, &w, 2);
+			if (buf[0] == 0xfe && buf[1] == 0xff)
+				curbuf->b_p_ubig = TRUE;
+			else
+				curbuf->b_p_ubig = FALSE;
+		}
+# endif
 	}
 #endif
 #ifdef FEPCTRL
@@ -490,6 +521,16 @@ set_init()
 #if defined(NT) && defined(USE_EXFILE) && defined(USE_MATOME)
 	curbuf->b_p_ec = strsave("");
 #endif
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+	curbuf->b_syn_ptr		= NULL;
+	curbuf->b_syn_tag		= NULL;
+	curbuf->b_syn_match		= NULL;
+	curbuf->b_syn_matchend	= NULL;
+	curbuf->b_syn_curp		= NULL;
+	curbuf->b_syn_line		= -1;
+	curbuf->b_syn_nline		= 0;
+	curbuf->b_syn_link		= NULL;
+#endif
 #ifdef NT
 	{
 		char p_win_init[32];
@@ -723,6 +764,7 @@ doset(arg)
 						for (buf = firstbuf; buf != NULL; buf = buf->b_next)
 						{
 							buf->b_p_tw = buf->b_p_tw_save;
+							buf->b_p_wm = buf->b_p_wm_save;
 							buf->b_p_ai = buf->b_p_ai_save;
 							buf->b_p_si = buf->b_p_si_save;
 						}
@@ -1546,6 +1588,9 @@ get_varp(p)
 		case PV_WM:		return (char_u *)&(curbuf->b_p_wm);
 #ifdef KANJI
 		case PV_JC:		return (char_u *)&(curbuf->b_p_jc);
+# ifdef UCODE
+		case PV_UB:		return (char_u *)&(curbuf->b_p_ubig);
+# endif
 #endif
 #ifdef FEPCTRL
 		case PV_FC:		return (char_u *)&(curbuf->b_p_fc);
@@ -1611,6 +1656,9 @@ buf_copy_options(bp_from, bp_to)
 	bp_to->b_p_tx = bp_from->b_p_tx;
 #ifdef KANJI
 	bp_to->b_p_jc = strsave(bp_from->b_p_jc);
+# ifdef UCODE
+	bp_to->b_p_ubig = bp_from->b_p_ubig;
+# endif
 #endif
 #ifdef FEPCTRL
 	bp_to->b_p_fc = bp_from->b_p_fc;
