@@ -183,6 +183,9 @@ static int ismult __ARGS((int));
 static void regjp __ARGS((int, char_u));
 static char * strjpchr __ARGS((char_u *, char_u, char_u));
 static char * mstrjpchr __ARGS((char_u *, char_u, char_u));
+#endif
+#ifndef notdef
+static char_u	* regstrext __ARGS((char_u *));
 #endif
 
 	static int
@@ -347,6 +350,9 @@ regcomp(exp)
 	int 			flags;
 /*	extern char    *malloc();*/
 
+#ifndef notdef
+	exp = regstrext(exp);
+#endif
 	if (exp == NULL) {
 		EMSG_RETURN(e_null);
 	}
@@ -684,7 +690,13 @@ regatom(flagp)
 				{
 					regparse++;
 					regc(S_CHAR);
-					regc(*regparse++);
+					if (ISkanji(*regparse))
+					{
+						regc(*regparse++);
+						regc(*regparse++);
+					}
+					else
+						regc(*regparse++);
 				}
 				else if ((ISkanji(*regparse)
 						&& regparse[2] == '-'
@@ -1350,11 +1362,22 @@ regmatch(prog)
 					return 0;
 				break;
 			}
+			else if (ISkana(reginput[0]))
+			{
+		  		if (regbol != NULL
+						&& ((reginput >= (regbol + 1)) && ISkana(reginput[-1]))
+						&& !((reginput >= (regbol + 2))
+							&& ISkanjiPointer(regbol, &reginput[-2]) == 1))
+					return 0;
+				break;
+			}
 			else
 			{
 		  		if (regbol != NULL && (reginput >= (regbol + 2))
 								&& ISkanjiPointer(regbol, &reginput[-2]) == 1)
 					break;
+		  		if (regbol != NULL && (reginput >= (regbol + 1)) && ISkana(reginput[-1]))
+					break;
 			}
 #endif
 		  	if (reginput != regbol && isidchar(reginput[-1]))
@@ -1377,10 +1400,19 @@ regmatch(prog)
 					return 0;
 				break;
 			}
+			else if (regbol != NULL && (reginput >= (regbol + 1))
+													&& ISkana(reginput[-1]))
+			{
+				if (ISkana(reginput[0]))
+					return 0;
+				break;
+			}
 			else
 			{
 		  		if (reginput[0] != NUL && ISkanji(reginput[0]))
 					break;
+		  		if (reginput[0] != NUL && ISkana(reginput[0]))
+					break;
 			}
 #endif
 		  	if (reginput == regbol || !isidchar(reginput[-1]))
@@ -2166,3 +2198,623 @@ mstrjpchr(s, c, k)
 	return NULL;
 }
 #endif
+
+#ifndef notdef
+static char_u	*
+regstrext(exp)
+	char_u		   *exp;
+{
+	char_u			*	p;
+	char_u			*	w;
+	static	char_u	*	exptop	= NULL;
+	int					size;
+	int					loop;
+	int					range;
+
+	if (exptop)
+	{
+		free(exptop);
+		exptop = NULL;
+	}
+	if (!reg_magic || exp == NULL)
+		return(exp);
+	for (p = exp, size = 0, range = FALSE, loop = 0; loop < 2;
+											p = exp, range = FALSE, loop++)
+	{
+		while (*p)
+		{
+# ifdef KANJI
+			if (ISkanji(*p))
+			{
+				if (loop)
+				{
+					*w++ = p[0];
+					*w++ = p[1];
+				}
+				p += 2;
+			}
+			else
+# endif
+			if (*p == '\\')
+			{
+# ifdef KANJI
+				if (ISkanji(p[1]))
+				{
+					if (loop)
+					{
+						*w++ = p[0];
+						*w++ = p[1];
+						*w++ = p[2];
+					}
+					p += 3;
+				}
+				else
+# endif
+				switch (p[1]) {
+				case 'e':				/* ESC */
+					if (loop == 0)
+						size += 1;
+					else
+						*w++ = '\033';
+					p += 2;
+					break;
+				case 't':				/* TAB */
+					if (loop == 0)
+						size += 1;
+					else
+						*w++ = '\t';
+					p += 2;
+					break;
+				case 'r':				/* CR */
+					if (loop == 0)
+						size += 1;
+					else
+						*w++ = '\015';
+					p += 2;
+					break;
+				case 'b':				/* BS */
+					if (loop == 0)
+						size += 1;
+					else
+						*w++ = '\010';
+					p += 2;
+					break;
+				case 'n':				/* NL */
+					if (loop == 0)
+						size += 1;
+					else
+						*w++ = '\012';
+					p += 2;
+					break;
+				case 'i':				/* identifier character */
+				case 'k':				/* keyword character */
+					if (loop == 0)
+						size += 10;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '9';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						*w++ = '_';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'I':				/* identifier but exluding digits */
+				case 'K':				/* keyword but exluding digits */
+					if (loop == 0)
+						size += 7;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						*w++ = '_';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'f':				/* filename character */
+					if (loop == 0)
+						size += 12;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = '\\';
+						*w++ = '\\';
+						*w++ = '/';
+						*w++ = ':';
+						*w++ = '\\';
+						*w++ = '*';
+						*w++ = '?';
+						*w++ = '"';
+						*w++ = '<';
+						*w++ = '>';
+						*w++ = '|';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'F':				/* filename but exluding digits */
+					if (loop == 0)
+						size += 15;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = '\\';
+						*w++ = '\\';
+						*w++ = '/';
+						*w++ = ':';
+						*w++ = '\\';
+						*w++ = '*';
+						*w++ = '?';
+						*w++ = '"';
+						*w++ = '<';
+						*w++ = '>';
+						*w++ = '|';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '9';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'p':				/* pritable character */
+					if (loop == 0)
+						size += 7;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '\t';
+						*w++ = '\040';
+						*w++ = '-';
+						*w++ = '\137';
+						*w++ = '\141';
+						*w++ = '-';
+						*w++ = '\176';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'P':				/* pritable but exluding digits */
+					if (loop == 0)
+						size += 7;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '\t';
+						*w++ = '\040';
+						*w++ = '-';
+						*w++ = '\057';
+						*w++ = '\072';
+						*w++ = '-';
+						*w++ = '\137';
+						*w++ = '\141';
+						*w++ = '-';
+						*w++ = '\176';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 's':				/* whitespace  SP/TAB */
+					if (loop == 0)
+						size += 2;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = ' ';
+						*w++ = '\t';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'S':				/* not whitespace */
+					if (loop == 0)
+						size += 3;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = ' ';
+						*w++ = '\t';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'd':				/* digit */
+					if (loop == 0)
+						size += 3;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '9';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'D':				/* not digit */
+					if (loop == 0)
+						size += 4;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '9';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'x':				/* hex digit */
+					if (loop == 0)
+						size += 9;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '9';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'f';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'X':				/* not hex digit */
+					if (loop == 0)
+						size += 10;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '9';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'f';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'o':				/* octal digit */
+					if (loop == 0)
+						size += 3;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '7';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'O':				/* not octal digit */
+					if (loop == 0)
+						size += 4;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '7';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'w':				/* word character */
+					if (loop == 0)
+						size += 10;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '9';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						*w++ = '_';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'W':				/* not word character */
+					if (loop == 0)
+						size += 11;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = '0';
+						*w++ = '-';
+						*w++ = '9';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						*w++ = '_';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'h':				/* head of word character */
+					if (loop == 0)
+						size += 7;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						*w++ = '_';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'H':				/* not head of word character */
+					if (loop == 0)
+						size += 8;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						*w++ = '_';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'a':				/* alphabetic character */
+					if (loop == 0)
+						size += 6;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'A':				/* not alphabetic character */
+					if (loop == 0)
+						size += 7;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!reg_ic)
+						{
+							*w++ = 'A';
+							*w++ = '-';
+							*w++ = 'F';
+						}
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'l':				/* lowercase character */
+					if (loop == 0)
+						size += 3;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'L':				/* not lowercase character */
+					if (loop == 0)
+						size += 4;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = 'a';
+						*w++ = '-';
+						*w++ = 'z';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'u':				/* uppercase character */
+					if (loop == 0)
+						size += 3;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = 'A';
+						*w++ = '-';
+						*w++ = 'Z';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case 'U':				/* not uppercase character */
+					if (loop == 0)
+						size += 4;
+					else
+					{
+						if (!range)
+							*w++ = '[';
+						*w++ = '^';
+						*w++ = 'A';
+						*w++ = '-';
+						*w++ = 'Z';
+						if (!range)
+							*w++ = ']';
+					}
+					p += 2;
+					break;
+				case '\0':
+					if (loop)
+						*w++ = p[0];
+					p++;
+					break;
+				default:
+					if (loop)
+					{
+						*w++ = p[0];
+						*w++ = p[1];
+					}
+					p += 2;
+					break;
+				}
+			}
+			else if (*p == '[')
+			{
+				range = TRUE;
+				if (loop)
+					*w++ = p[0];
+				p++;
+			}
+			else if (*p == ']')
+			{
+				range = FALSE;
+				if (loop)
+					*w++ = p[0];
+				p++;
+			}
+			else
+			{
+				if (loop)
+					*w++ = p[0];
+				p++;
+			}
+		}
+		if (loop)
+			*w = '\0';
+		else
+		{
+			if (size == 0)
+				return(exp);
+			if ((w = alloc(strlen(exp) + 1 + size)) == NULL)
+				return(NULL);
+			exptop = w;
+		}
+	}
+	return(exptop);
+}
+#endif
