@@ -46,6 +46,7 @@ static int		invert = 0;				/* set to INVERTCODE when inverting */
 # define SCREEN(scr)	  ((scr)[Columns])
 # define SCRTST(scr)	  (((scr)[Columns]) != 0)
 # define SCRCMP(scr, inv) (inv == 0 ? (((scr)[Columns]) == 0) : (((scr)[Columns]) != 0))
+# define SCRINV(scr, inv) (inv == 0 ? (((scr)[Columns]) != 0) : (((scr)[Columns]) != inv))
 #endif
 #if defined(KANJI) && defined(NT)
 static int		color = 0;
@@ -586,6 +587,9 @@ win_line(wp, lnum, startrow, endrow)
 	int				row;				/* row in the window, excluding w_winpos */
 	int				screen_row;			/* row on the screen, including w_winpos */
 	char_u			*ptr;
+#ifdef KANJI
+	char_u			*ltop;
+#endif
 	char_u			extra[16];			/* "%ld" must fit in here */
 	char_u			*p_extra;
 	int 			n_extra;
@@ -628,7 +632,7 @@ win_line(wp, lnum, startrow, endrow)
 				{
 					FPOS	w;
 					w.lnum = lnum;
-					w.col = vcol2col(curwin, lnum, fromcol, NULL);
+					w.col = vcol2col(curwin, lnum, fromcol, NULL, 0, 0);
 					if (*ml_get_pos(&w) != '\t')
 						fromcol = getvcol(wp, &w, 2);
 				}
@@ -645,7 +649,7 @@ win_line(wp, lnum, startrow, endrow)
 					{
 						FPOS	w;
 						w.lnum = lnum;
-						w.col = vcol2col(curwin, lnum, tocol, NULL);
+						w.col = vcol2col(curwin, lnum, tocol, NULL, 0, 0);
 						if (*ml_get_pos(&w) != '\t')
 							tocol = getvcol(wp, &w, 3);
 					}
@@ -685,11 +689,19 @@ win_line(wp, lnum, startrow, endrow)
 			canopt = FALSE;
 	}
 
+#ifdef KANJI
+	ltop =
+#endif
 	ptr = ml_get_buf(wp->w_buffer, lnum, FALSE);
 	if (!wp->w_p_wrap)		/* advance to first character to be displayed */
 	{
 		while (vcol < wp->w_leftcol && *ptr)
+		{
 #ifdef KANJI
+# if defined(NT) && defined(SYNTAX)
+			if (p_syn && GuiWin)
+				is_syntax(wp, lnum, &ltop, &ptr);
+# endif
 			if (ISkanji(*ptr))
 			{
 				vcol += 2;
@@ -698,6 +710,7 @@ win_line(wp, lnum, startrow, endrow)
 			else
 #endif
 			vcol += chartabsize(*ptr++, vcol);
+		}
 		if (vcol > wp->w_leftcol)
 		{
 			n_spaces = vcol - wp->w_leftcol;	/* begin with some spaces */
@@ -724,13 +737,16 @@ win_line(wp, lnum, startrow, endrow)
 			if (((vcol == fromcol && !(noinvcur && vcol == wp->w_virtcol)) ||
 #ifdef KANJI
 					(noinvcur
-						&& (ptr > ml_get_buf(wp->w_buffer, lnum, FALSE))
-						&& ISkanjiPointer(ml_get_buf(wp->w_buffer, lnum, FALSE), ptr -1) == 2
+						&& (ptr > ltop)
+						&& ISkanjiPointer(ml_get_buf(wp->w_buffer, lnum, FALSE), ptr - 1) == 2
 						&& vcol == wp->w_virtcol + 2 && vcol >= fromcol) ||
 #endif
 					(noinvcur && vcol == wp->w_virtcol + 1 && vcol >= fromcol)) &&
 					vcol < tocol)
+			{
+				(void)set_highlight('v');
 				start_highlight();		/* start highlighting */
+			}
 			else if (invert && (vcol == tocol || (noinvcur && vcol == wp->w_virtcol)))
 				stop_highlight();		/* stop highlighting */
 		}
@@ -753,6 +769,25 @@ win_line(wp, lnum, startrow, endrow)
 		}
 		else
 		{
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+			if (p_syn && GuiWin)
+			{
+				if (!canopt && (fromcol <= vcol) && (vcol <= tocol))
+					;
+				else
+				{
+					int		clr = is_syntax(wp, lnum, &ltop, &ptr);
+
+					if (clr)
+					{
+						color = clr;
+						start_highlight();		/* start highlighting */
+					}
+					else if (!('a' <= color && color <= 'z'))
+						stop_highlight();		/* stop highlighting */
+				}
+			}
+#endif
 #ifdef KANJI
 			if ((c = *ptr++) < ' ' || c == DEL || (c >=0x80 && !ISdisp(c)))
 #else
@@ -774,13 +809,43 @@ win_line(wp, lnum, startrow, endrow)
 					p_extra = (char_u *)"";
 					n_extra = 1;
 					c = '$';
+#if defined(KANJI) && defined(NT) && defined(SYNTAX)
+					if (p_syn && GuiWin)
+					{
+						int		clr = is_crsyntax(wp);
+
+						if (clr)
+						{
+							color = clr;
+							start_highlight();		/* start highlighting */
+						}
+						else if (!('a' <= color && color <= 'z'))
+							stop_highlight();		/* stop highlighting */
+					}
+#endif
 				}
 #ifdef CRMARK
 				else if (c == NUL && wp->w_p_cr)
 				{
 					p_extra = (char_u *)"";
 					n_extra = 1;
-					c = '!';
+					c = *p_cc;
+					if (c == NUL || ISkanji(c) || c <= ' ')
+						c = '$';
+# if defined(KANJI) && defined(NT) && defined(SYNTAX)
+					if (p_syn && GuiWin)
+					{
+						int		clr = is_crsyntax(wp);
+
+						if (clr)
+						{
+							color = clr;
+							start_highlight();		/* start highlighting */
+						}
+						else if (!('a' <= color && color <= 'z'))
+							stop_highlight();		/* stop highlighting */
+					}
+# endif
 				}
 #endif
 				else if (c != NUL)
@@ -799,7 +864,7 @@ win_line(wp, lnum, startrow, endrow)
 				if (vcol == 0)	/* invert first char of empty line */
 				{
 #ifdef KANJI
-					if (*screenp != ' ' || !SCRTST(screenp))
+					if (*screenp != ' ' || SCRINV(screenp, invert))
 					{
 							*screenp = ' ';
 							SCREEN(screenp) = invert;
@@ -841,9 +906,9 @@ win_line(wp, lnum, startrow, endrow)
 #ifdef KANJI
 		if (col >= (ISkanji(c) ? Columns - 1 : Columns)) /* continuous line */
 		{
-			if (col == Columns - 1 && (*screenp != '\\' || !SCRCMP(screenp, invert)))
+			if (col == Columns - 1 && (*screenp != BRCHAR || !SCRCMP(screenp, invert)))
 			{
-				*screenp = '\\';
+				*screenp = BRCHAR;
 				SCREEN(screenp) = invert;
 				screen_char(screenp, screen_row, col);
 			}
@@ -888,8 +953,7 @@ win_line(wp, lnum, startrow, endrow)
 
 			c1 = c;
 			c2 = *ptr++;
-			if (*screenp != c1 || *(screenp + 1) != c2
-					|| (invert ? !SCRTST(screenp) : SCRTST(screenp)))
+			if (*screenp != c1 || *(screenp + 1) != c2 || SCRINV(screenp, invert))
 			{
 				if (invert)
 				{
@@ -916,7 +980,7 @@ win_line(wp, lnum, startrow, endrow)
 		}
 		else
 		{
-			if (!SCRCMP(screenp, invert) || *screenp != c)
+			if (SCRINV(screenp, invert) || *screenp != c)
 			{
 				*screenp = c;
 				SCREEN(screenp) = invert;
@@ -1118,7 +1182,9 @@ screen_char(p, row, col)
 		int 	row;
 		int 	col;
 {
+#ifndef KANJI
 	int			c;
+#endif
 	int			noinvcurs;
 
 	/*
@@ -1285,7 +1351,7 @@ screen_fill(start_row, end_row, start_col, end_col, c1, c2)
 		for (col = start_col; col < end_col; ++col)
 		{
 #ifdef KANJI
-			if (*screenp != c || !SCRCMP(screenp,invert))
+			if (*screenp != c || !SCRCMP(screenp, invert))
 #else
 			if (*screenp != c)
 #endif
@@ -1743,6 +1809,10 @@ getvcol(wp, pos, type)
 	int				c;
 
 	vcol = 0;
+#ifdef KANJI
+	if (type == 99)
+		vcol = (curwin->w_p_nu ? 8 : 0);
+#endif
 	ptr = ml_get_buf(wp->w_buffer, pos->lnum, FALSE);
 	for (col = pos->col; col >= 0; --col)
 	{
@@ -1758,14 +1828,27 @@ getvcol(wp, pos, type)
 		/* A tab gets expanded, depending on the current column */
 #ifdef KANJI
 		if (ISkanji(c))
+		{
+			if (type == 99 && (vcol % Columns) == (Columns - 1))
+				vcol++;
 			incr = 2;
+		}
 		else
 #endif
 		incr = chartabsize(c, (long)vcol);
 
+#ifdef KANJI
+		if (type == 99 && *ptr == NUL && c < ' ' && c != TAB
+										&& (vcol % Columns) == (Columns - 1))
+			incr = 1;
+#endif
 		if (col == 0)		/* character at pos.col */
 		{
+#ifdef KANJI
+			if (type == 3 || ((type == 1 || type == 99) && c == TAB && State == NORMAL && !wp->w_p_list))
+#else
 			if (type == 3 || (type == 1 && c == TAB && State == NORMAL && !wp->w_p_list))
+#endif
 				--incr;
 			else
 				break;
