@@ -303,7 +303,7 @@ char_u		**	matchend;
 					cp++;
 				else if (ISkanji(*cp))
 				{
-					if (cp[0] == 0x81 && cp[1] == 0x40)
+					if (isjpspace(cp[0], cp[1]))
 						cp += 2;
 					else
 						break;
@@ -329,7 +329,7 @@ char_u		**	matchend;
 					if (ep == NULL)
 						ep = cp;
 				}
-				else if (ISkanji(*cp) && cp[0] == 0x81 && cp[1] == 0x40)
+				else if (ISkanji(*cp) && isjpspace(cp[0], cp[1]))
 				{
 					if (ep == NULL)
 						ep = cp;
@@ -1137,6 +1137,9 @@ showmatch(initc)
 	int				do_quotes;			/* check for quotes in current line */
 	int				hash_dir = 0;		/* Direction searched for # things */
 	int				comment_dir = 0;	/* Direction searched for comments */
+#ifdef USE_OPT
+	int				incomment = 0;		/* non-zero when inside block comment */
+#endif
 
 	pos = curwin->w_cursor;
 	linep = ml_get(pos.lnum); 
@@ -1307,9 +1310,6 @@ showmatch(initc)
 		 * Go to the next position, forward or backward. We could use
 		 * inc() and dec() here, but that is much slower
 		 */
-#ifdef KANJI
-skip:
-#endif
 		if (idx)              			/* backward search */
 		{
 			if (pos.col == 0)   		/* at start of line, go to previous one */
@@ -1330,7 +1330,35 @@ skip:
 			if (linep[pos.col] && ISkanjiPosition(linep, pos.col + 1))
 			{
 				pos.col--;
-				goto skip;
+				continue;
+			}
+#endif
+#ifdef USE_OPT
+			if (!comment_dir && !inquote && (curbuf->b_p_opt & FOPT_C_COMMENT))
+			{
+				char_u		k = linep[pos.col];
+				char_u	*	s;
+				char_u	*	c;
+
+				linep[pos.col] = NUL;
+				s = strrchr(linep, '"');
+				c = strrchr(linep, '/');
+				linep[pos.col] = k;
+				/* line comment */
+				if (!incomment && c > s && linep != c && c[-1] == '/')
+				{
+					pos.col = &c[-1] - linep;
+					continue;
+				}
+				/* block comment */
+				if (!incomment && c > s && linep != c && c[-1] == '*')
+					incomment = -1;
+				if (incomment && c > s && c[1] == '*')
+				{
+					incomment = 0;
+					pos.col = c - linep;
+					continue;
+				}
 			}
 #endif
 		}
@@ -1354,7 +1382,23 @@ skip:
 			if (ISkanjiPosition(linep, pos.col + 1))
 			{
 				pos.col++;
-				goto skip;
+				continue;
+			}
+#endif
+#ifdef USE_OPT
+			if (!comment_dir && !inquote && (curbuf->b_p_opt & FOPT_C_COMMENT))
+			{
+				/* line comment */
+				if (!incomment && linep[pos.col] == '/' && linep[pos.col + 1] == '/')
+				{
+					pos.col = STRLEN(linep);
+					continue;
+				}
+				/* block comment */
+				if (!incomment && linep[pos.col] == '/' && linep[pos.col + 1] == '*')
+					incomment = 1;
+				if (incomment && linep[pos.col] == '*' && linep[pos.col + 1] == '/')
+					incomment = 0;
 			}
 #endif
 		}
@@ -1420,6 +1464,11 @@ skip:
 		 * unless this line or the previous one ends in a '\'.
 		 * Complicated, isn't it?
 		 */
+#ifdef USE_OPT
+		if (incomment && (curbuf->b_p_opt & FOPT_C_COMMENT))
+			;
+		else
+#endif
 		switch (c = linep[pos.col])
 		{
 		case NUL:
@@ -1770,7 +1819,7 @@ cls()
 			return 1;
 		if ((stype != 0) && (p_ww & 32))
 			return JPC_HIRA;
-		if ((ret = jpcls(c, *(ml_get_cursor() + 1))) >= 0)
+		if ((ret = jpcls((char_u)c, *(ml_get_cursor() + 1))) >= 0)
 		{
 			if (ret != JPC_KIGOU && stype != 0)
 				return JPC_HIRA;
