@@ -503,7 +503,7 @@ flushbuf()
 			char_u *tmpptr;
 
 			outbuf[bpos] = NUL;
-			tmpptr = kanjiconvsto(outbuf, JP_DISP);
+			tmpptr = kanjiconvsto(outbuf, JP_DISP, TRUE);
 			mch_write(tmpptr, STRLEN(tmpptr));
 			free(tmpptr);
 		}
@@ -557,7 +557,6 @@ outchar2(c1, c2)
 }
 #endif
 
-/*
 /*
  * a never-padding outstr.
  * use this whenever you don't want to run the string through tputs.
