@@ -30,10 +30,10 @@
 
 #ifndef USE_SYSTEM		/* use fork/exec to start the shell */
 # include <sys/wait.h>
-# if !defined(SCO) && !defined(SOLARIS) && !defined(hpux) && !defined(__NetBSD__) && !defined(__FreeBSD__) && !defined(_SEQUENT_) && !defined(UNISYS) && !defined(__sgi) && !defined(AIX)	/* SCO returns pid_t */
+# if !defined(SCO) && !defined(SOLARIS) && !defined(hpux) && !defined(__NetBSD__) && !defined(__FreeBSD__) && !defined(_SEQUENT_) && !defined(UNISYS) && !defined(__sgi) && !defined(AIX) && !defined(__bsdi__)	/* SCO returns pid_t */
 extern int fork();
 # endif
-# if !defined(linux) && !defined(SOLARIS) && !defined(USL) && !defined(sun) && !(defined(hpux) && defined(__STDC__)) && !defined(__NetBSD__) && !defined(__FreeBSD__) && !defined(USL) && !defined(UNISYS) && !defined(sony) && !defined(__sgi) && !defined(AIX) && !defined(__CYGWIN__)
+# if !defined(linux) && !defined(SOLARIS) && !defined(USL) && !defined(sun) && !(defined(hpux) && defined(__STDC__)) && !defined(__OpenBSD__) && !defined(__NetBSD__) && !defined(__FreeBSD__) && !defined(USL) && !defined(UNISYS) && !defined(sony) && !defined(__sgi) && !defined(AIX) && !defined(__CYGWIN__) && !defined(__bsdi__)
 extern int execvp __ARGS((const char *, const char **));
 # endif
 #endif
@@ -557,6 +557,7 @@ mch_settitle(title, icon)
 	 */
 	if (	STRCMP(term_strings.t_name, "xterm") == 0 ||
 #ifdef KANJI
+			STRCMP(term_strings.t_name, "rxvt" ) == 0 ||
 			STRCMP(term_strings.t_name, "kterm") == 0 ||
 #endif
 			STRCMP(term_strings.t_name, "builtin_xterm") == 0)
