@@ -51,17 +51,17 @@
  */
 
 /* #include <stdio.h> */
-#ifdef	WIN32			/*Tom_W*/
-#include <windows.h>
-# ifdef USE_EXFILE
-#  include "exfile.h"
-# endif
-#else
-#include <dos.h>
-#endif
 #include <string.h>
 #include <io.h>
 #include <stdlib.h>
+#ifdef	WIN32			/*Tom_W*/
+# include <windows.h>
+#  ifdef USE_EXFILE
+#   include "exfile.h"
+#  endif
+#else
+# include <dos.h>
+#endif
 
 #define PSP	_psp
 
@@ -519,11 +519,15 @@ static int ustrcmp(const void *s1, const void *s2)
 	unsigned char * p1 = *(unsigned char **)s1;
 	unsigned char * p2 = *(unsigned char **)s2;
 
+# if 0
 	while ((*p1 != '\0') && (*p1 == *p2)) {
 		p1++;
 		p2++;
 	}
 	return *p1 - *p2;
+# else
+	return (stricmp(p1, p2));
+# endif
 }
 #endif
 
